# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
#import StringIO
from lxml import etree
from pytrainer.lib.date import getDateTime

class gpxplusNokia():
    def __init__(self, parent = None, data_path = None):
        self.parent = parent
        self.pytrainer_main = parent.parent
        self.tmpdir = self.pytrainer_main.profile.tmpdir
        self.main_data_path = data_path
        self.data_path = os.path.dirname(__file__)
        self.xmldoc = None
        self.activitiesSummary = []

    def getXmldoc(self):
        ''' Function to return parsed xmlfile '''
        return self.xmldoc

    def getFileType(self):
        return _("Nokia Export - GPS eXchange file")

    def getActivitiesSummary(self):
        return self.activitiesSummary

    def testFile(self, filename):
        logging.debug('>>')
        logging.debug("Testing " + filename)
        #Check if file is a GPX
        try:
            #parse as xml
            xmldoc = etree.parse(filename)
            #Parse XML schema
            xmlschema_doc = etree.parse(self.main_data_path+"schemas/Topografix_gpx11-Nokia.xsd")
            xmlschema = etree.XMLSchema(xmlschema_doc)
            if (xmlschema.validate(xmldoc)):
                #Valid gpx file
                self.xmldoc = xmldoc
                startTime = getDateTime(self.startTimeFromFile(xmldoc))
                indatabase = self.inDatabase(xmldoc, startTime)
                sport = self.getSport(xmldoc)
                duration  = self.getDetails(xmldoc, startTime)
                distance = ""
                self.activitiesSummary.append( (0,
                                                indatabase,
                                                startTime[1].strftime("%Y-%m-%dT%H:%M:%S"),
                                                distance ,
                                                str(duration),
                                                sport,
                                                ) )
                return True
        except:
            #Not gpx file
            return False
        return False

    def inDatabase(self, tree, startTime):
        #comparing date and start time (sport may have been changed in DB after import)
        time = startTime
        if time is None:
            return False
        time = time[0].strftime("%Y-%m-%dT%H:%M:%SZ")
        if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
            return True
        else:
            return False

    def getDetails(self, tree, startTime):
        root = tree.getroot()
        #Get all times from file
        times = root.findall(".//{http://www.topografix.com/GPX/1/1}time")
        time = times[-1].text
        return getDateTime(time)[0]-startTime[0]

    def getSport(self, tree):
        #No sport in GPX file
        root = tree.getroot()
        element = root.find(".//{http://www.topografix.com/GPX/1/1}metadata/{http://www.topografix.com/GPX/1/1}desc")
        if element is not None:
                return element.text
        return None

    def startTimeFromFile(self, tree):
        """ Function to return the first time element from a GPX 1.1 file """
        root = tree.getroot()
        timeElement = root.find(".//{http://www.topografix.com/GPX/1/1}time")
        if timeElement is not None:
                return timeElement.text
        return None

    def getGPXFile(self, ID, file_id):
        """
            Generate GPX file based on activity ID

            Returns (sport, GPX filename)
        """
        sport = None
        gpxFile = None
        if ID == "0": #Only one activity in file
            gpxFile = "%s/gpx-%s-%s.gpx" % (self.tmpdir, file_id, ID)
            sport = self.getSport(self.xmldoc)
            self.createGPXfile(gpxFile, self.xmldoc)
        return sport, gpxFile

    def createGPXfile(self, gpxfile, tree):
        tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from sqlalchemy.sql.expression import text
import logging

class _SportNormalizer(object):
    
    """A row from the sport DB table that can be "normalized" to have invalid
    values replaced with valid defaults."""
    
    def __init__(self, id, weight, color, met, max_pace):
        self._id = id
        self._weight = weight
        self._color =  color
        self._met = met
        self._max_pace = max_pace
        
    def normalize(self, migrate_engine):
        self._normalize_weight(migrate_engine)
        self._normalize_color(migrate_engine)
        self._normalize_met(migrate_engine)
        self._normalize_max_pace(migrate_engine)
            
    def _normalize_weight(self, migrate_engine):
        valid = True
        try:
            weight = float(self._weight)
            if weight < 0:
                valid = False
        except:
            valid = False
        if not valid:
            logging.debug("Sport with id '%s' has invalid weight: '%s'. Replacing with default.", self._id, self._weight)
            migrate_engine.execute(text("update sports set weight=:weight where id_sports=:id"), id=self._id, weight=0.0)
            
    def _normalize_color(self, migrate_engine):
        try:
            # colors that do not have exactly 6 hexadecimal digits should also
            # be invalid, but we do not expect such invalid values to exist.
            int(self._color, 16)
        except:
            logging.debug("Sport with id '%s' has invalid color: '%s'. Replacing with default.", self._id, self._color)
            migrate_engine.execute(text("update sports set color=:color where id_sports=:id"), id=self._id, color="0000ff")
            
    def _normalize_met(self, migrate_engine):
        valid = True
        if self._met is not None:
            try:
                met = float(self._met)
                if met < 0:
                    valid = False
            except:
                valid = False
        if not valid:
            logging.debug("Sport with id '%s' has invalid MET: '%s'. Replacing with default.", self._id, self._met)
            migrate_engine.execute(text("update sports set met=:met where id_sports=:id"), id=self._id, met=None)
            
    def _normalize_max_pace(self, migrate_engine):
        valid = True
        if self._max_pace is not None:
            try:
                max_pace = int(self._max_pace)
                if max_pace <= 0:
                    valid = False
            except:
                valid = False
        if not valid:
            logging.debug("Sport with id '%s' has invalid max pace: '%s'. Replacing with default.", self._id, self._max_pace)
            migrate_engine.execute(text("update sports set max_pace=:max_pace where id_sports=:id"), id=self._id, max_pace=None)

def upgrade(migrate_engine):
    sport_rows = migrate_engine.execute("select id_sports, weight, color, met, max_pace from sports")
    sport_normalizers = []
    for (id, weight, color, met, max_pace) in sport_rows:
        sport_normalizers.append(_SportNormalizer(id, weight, color, met, max_pace))
    for sport_normalizer in sport_normalizers:
        sport_normalizer.normalize(migrate_engine)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import division
import os
import logging
import traceback
import gtk, gobject
from SimpleGladeApp import SimpleBuilderApp
from windowcalendar import WindowCalendar

from pytrainer.lib.date import getLocalTZ, time2second
import dateutil.parser
from dateutil.tz import tzutc
from pytrainer.lib.uc import UC

class WindowRecord(SimpleBuilderApp):
    def __init__(self, equipment_service, data_path = None, listSport = None, parent = None, date = None, title=None, distance=None, time=None, upositive=None, unegative=None, bpm=None, calories=None, comment=None, windowTitle=None, equipment=[]):
        logging.debug(">>")
        self.parent = parent
        self.pytrainer_main = parent.pytrainer_main
        self.uc = UC()
        logging.debug("Using US system: "+ str(self.uc.us))
        self.data_path = data_path
        self.mode = "newrecord"
        self.id_record = ""
        self.store = None
        self.active_row = None
        self.activity_data = [] 
        SimpleBuilderApp.__init__(self, "newrecord.ui")
        self.conf_options = [
            "rcd_date",
            "rcd_sport",
            "rcd_distance",
            "rcd_beats",
            "rcd_comments",
            "rcd_average",
            "rcd_calories",
            "rcd_title",
            "rcd_gpxfile",
            "rcd_upositive",
            "rcd_unegative",
            "rcd_maxbeats",
            "rcd_pace",
            "rcd_maxpace",
            "rcd_maxvel",
            ]
        self.listSport = {}
        for sport in listSport:
            self.listSport[sport.id] = sport.name
        for i in self.listSport:    
            self.rcd_sport.insert_text(i,self.listSport[i])
        self.rcd_sport.set_active(0)

        if windowTitle is not None:
            self.newrecord.set_title(windowTitle)
        if date != None:
            self.setDate(date)
        if title != None:
            self.rcd_title.set_text(title)
        if distance != None:
            #self.rcd_distance.set_text(distance)
            #myset_text(rcd_distance, 'distance', distance, us=self.us, round=2)
            self.rcd_distance.set_text(self.uc.distance(distance))
        if time != None:
            self.setTime(time)
        if distance!=None and time!=None:
            self.on_calcavs_clicked(None)
        if upositive != None:
            self.rcd_upositive.set_text(self.uc.height(upositive))
        if unegative != None:
            self.rcd_unegative.set_text(self.uc.height(unegative))
        if calories != None:
            self.rcd_calories.set_text(calories)
            
        #populate labels with units        
        self.label_rcd_distance.set_text(_('Distance') + ' (%s)' %self.uc.unit_distance)
        self.label_rcd_maxvel.set_text(_('Max') + ' (%s)' %self.uc.unit_speed)
        self.label_rcd_average.set_text(_('Average') + ' (%s)' %self.uc.unit_speed)
        self.label_rcd_maxpace.set_text(_('Max') + ' (%s)' %self.uc.unit_pace)
        self.label_rcd_pace.set_text(_('Pace') + ' (%s)' %self.uc.unit_pace)
        self.label_rcd_upositive.set_text(_('Ascent') + ' (%s)' %self.uc.unit_height)
        self.label_rcd_unegative.set_text(_('Descent') + ' (%s)' %self.uc.unit_height)        

        self._init_equipment(equipment, equipment_service)
        logging.debug("<<")
        
    def _init_equipment(self, selected_equipment, equipment_service):
        equipment = {}
        active_equipment = equipment_service.get_active_equipment()
        for item in active_equipment:
            equipment[item] = False
        if len(active_equipment) == 0:
            self.noActiveEquipmentMessageContainer.set_visible(True)
        for item in selected_equipment:
            equipment[item] = True
        list_store = gtk.ListStore(int, str, bool)
        for item in equipment:
            list_store.append((item.id, item.description, equipment[item]))
        tree_view = self.treeviewRecordEquipment
        cell_renderer = gtk.CellRendererToggle()
        cell_renderer.connect('toggled', self._equipment_selection_toggled)
        tree_view.append_column(gtk.TreeViewColumn("Selected", cell_renderer, active=2))
        tree_view.append_column(gtk.TreeViewColumn("Equipment Item", gtk.CellRendererText(), text=1))
        tree_view.set_model(list_store)
    
    def _equipment_selection_toggled(self, widget, path):
        list_store = self.treeviewRecordEquipment.get_model()
        iter = list_store.get_iter(path)
        value = list_store.get_value(iter, 2)
        list_store.set(iter, 2, not value)
    
    def _get_selected_equipment_ids(self):
        selected_ids = []
        list_store = self.treeviewRecordEquipment.get_model()
        iter = list_store.get_iter_first()
        while iter is not None:
            id = list_store.get_value(iter, 0)
            selected = list_store.get_value(iter, 2)
            if selected:
                selected_ids.append(id)
            iter = list_store.iter_next(iter)
        return selected_ids
    
    def getActivityData(self):
        return self.activity_data
        
    def populateMultiWindow(self, activities):
        logging.debug(">>")
        self.mode = "multiple_activities"
        #activities (activity_id, start_time, distance, duration, sport, gpx_file, file_id)
        self.activity_data = []
        #Make treeview
        self.store = self.build_tree_view()
        #Add data
        for activity in activities:
            iter = self.store.append()
            self.store.set(
                    iter,
                    0, activity[0],
                    1, activity[1],
                    2, activity[2],
                    3, activity[3],
                    4, activity[4],
                    5, activity[5]
                    )
            details = {}
            details["complete"]  = False
            details["rcd_distance"] = activity[2]
            duration = activity[3]
            if duration.count(":") == 2:
                hours, mins, secs = duration.split(":")
            else:
                hours = mins = secs = 0
            #details["rcd_hour"] = float(hours)
            #details["rcd_min"] = float(mins)
            #details["rcd_second"] = float(secs)
            #details["rcd_time"] = (((float(hours) * 60) + float(mins)) * 60) + float(secs)
            details["activity_id"] = activity[0]
            details["rcd_time"] = (float(hours), float(mins), float(secs))
            details["rcd_sport"] = activity[4]
            details["rcd_gpxfile"] = activity[5]
            details["file_id"] = activity[6]
            self.activity_data.append(details)
        self.scrolledwindowEntries.show_all()
        #Hide some of the buttons
        self.button25.hide() #GPX file "Open" button
        self.button24.hide() #GPX file "Calculate Values" button
        self.button10.hide() #Distance "Calculate" button
        self.button11.hide() #Duration "Calculate" button
        #self.button12.hide() #Velocity "Calculate" button
        self.button43.hide() #Pace "Calculate" button
        #Make GPX file 'unsensitive'
        self.rcd_gpxfile.set_sensitive(0)       
        while gtk.events_pending(): # This allows the GUI to update 
            gtk.main_iteration()    # before completion of this entire action
        #Select first row and display details
        self.treeviewEntries.set_cursor(0)  
        self.show_treeviewEntries_row(0)
        logging.debug("<<")
        
    def build_tree_view(self):
        store = gtk.ListStore(  gobject.TYPE_STRING,
                                gobject.TYPE_STRING,
                                gobject.TYPE_STRING,
                                gobject.TYPE_STRING, 
                                gobject.TYPE_STRING, 
                                gobject.TYPE_STRING )
        column_names=["id", _("Start Time"), _("Distance"),_("Duration"),_("Sport"), _("GPX File")]
        for column_index, column_name in enumerate(column_names):
            #Add columns
            column = gtk.TreeViewColumn(column_name, gtk.CellRendererText(), text=column_index)
            column.set_sort_column_id(column_index)
            if column_name == "id":
                column.set_visible(False)
            column.set_resizable(True)
            self.treeviewEntries.append_column(column)
        self.treeviewEntries.set_headers_clickable(True)
        self.treeviewEntries.set_model(store)
        return store
        
    def on_accept_clicked(self,widget):
        logging.debug(">>")
        if self.mode == "multiple_activities":
            logging.debug("multiple_activities")
            #Check for edited data in comments
            if self.active_row is not None:
                buffer = self.rcd_comments.get_buffer()
                start,end = buffer.get_bounds()
                comments = buffer.get_text(start,end, True).replace("\"","'")
                self.activity_data[self.active_row]["rcd_comments"] = comments
                #Advanced tab items
                self.activity_data[self.active_row]["rcd_maxpace"] = self.rcd_maxpace.get_text()
                self.activity_data[self.active_row]["rcd_pace"] = self.rcd_pace.get_text()
                self.activity_data[self.active_row]["rcd_upositive"] = self.rcd_upositive.get_text()
                self.activity_data[self.active_row]["rcd_unegative"] = self.rcd_unegative.get_text()
                self.activity_data[self.active_row]["rcd_maxbeats"] = self.rcd_maxbeats.get_text()
                self.activity_data[self.active_row]["rcd_beats"] = self.rcd_beats.get_text()
                self.activity_data[self.active_row]["rcd_calories"] = self.rcd_calories.get_text()
            row = 0 
            for activity in self.activity_data:
                index = self.activity_data.index(activity)
                if activity["complete"] is False:
                    #Did not view or modify this record - need to get all the details
                    print "Activity incomplete.. " + activity["rcd_gpxfile"]
                    self.update_activity_data(row, activity["rcd_gpxfile"], activity["rcd_sport"])
                activity["rcd_title"] = activity["rcd_title"].replace("\"","'")
                #Add activity to DB etc
                laps = activity.pop("laps", ())
                selected_equipment_ids = self._get_selected_equipment_ids()
                self.activity_data[index]["db_id"] = self.parent.insertRecord(activity, laps, equipment=selected_equipment_ids)
                row += 1
            logging.debug("Processed %d rows of activity data" % row)
        else:
            logging.debug("Single activity")
            list_options = {}
            trackSummary = {}
            list_options["rcd_date"] = self.rcd_date.get_text()
            list_options["rcd_sport"] = self.rcd_sport.get_active_text()
            list_options["rcd_distance"] = self.uc.usr2sys_str('distance', self.rcd_distance.get_text())
            list_options["rcd_beats"] = self.rcd_beats.get_text()
            list_options["rcd_average"] = self.uc.usr2sys_str('speed', self.rcd_average.get_text())
            list_options["rcd_calories"] = self.rcd_calories.get_text()
            list_options["rcd_title"] = self.rcd_title.get_text().replace("\"","'")
            list_options["rcd_gpxfile"] = self.rcd_gpxfile.get_text()
            list_options["rcd_upositive"] = self.uc.usr2sys_str('height', self.rcd_upositive.get_text())
            list_options["rcd_unegative"] = self.uc.usr2sys_str('height', self.rcd_unegative.get_text())
            list_options["rcd_maxbeats"] = self.rcd_maxbeats.get_text()
            # 2011.11.05 - dgranda
            # Pace is shown to user in mm:ss format
            # But pace in database is stored in a mixed way: 4.30 for 4:30 (4.5 in 'decimal'). This needs to be changed!!
            list_options["rcd_pace"] = self.uc.usr2sys_str('pace', self.rcd_pace.get_text().replace(":","."))
            list_options["rcd_maxpace"] = self.uc.usr2sys_str('pace', self.rcd_maxpace.get_text().replace(":","."))
            list_options["rcd_maxvel"] = self.uc.usr2sys_str('speed', self.rcd_maxvel.get_text())
            list_options["rcd_time"] = [self.rcd_hour.get_value_as_int(),self.rcd_min.get_value_as_int(),self.rcd_second.get_value_as_int()]
            buffer = self.rcd_comments.get_buffer()
            start,end = buffer.get_bounds()
            comment = buffer.get_text(start,end, True)
            list_options["rcd_comments"] = comment.replace("\"","'")
            selected_equipment_ids = self._get_selected_equipment_ids()
            # Added to change start time, only activities without GPX+ source file - dgranda 2011/06/10
            record_time = self.rcd_starttime.get_text()
            record_date = self.rcd_date.get_text()
            localtz = getLocalTZ()
            date = dateutil.parser.parse(record_date+" "+record_time+" "+localtz)
            local_date = str(date)
            utc_date = date.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%SZ")
            list_options["date_time_utc"] = utc_date
            list_options["date_time_local"] = local_date

            if self.mode == "newrecord":
                logging.debug('Track data: '+str(list_options))
                if list_options["rcd_gpxfile"] != "":
                    logging.info('Adding new activity based on GPX file')
                    self.parent.insertRecord(list_options, None, selected_equipment_ids)
                else:
                    logging.info('Adding new activity based on provided data')
                    #Manual entry, calculate time info
                    record_time = self.rcd_starttime.get_text()
                    record_date = self.rcd_date.get_text()
                    localtz = getLocalTZ()
                    date = dateutil.parser.parse(record_date+" "+record_time+" "+localtz)
                    local_date = str(date)
                    utc_date = date.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%SZ")
                    list_options["date_time_utc"] = utc_date
                    list_options["date_time_local"] = local_date
                    self.parent.insertRecord(list_options, equipment=selected_equipment_ids)
            elif self.mode == "editrecord":
                self.parent.updateRecord(list_options, self.id_record, equipment=selected_equipment_ids)
        logging.debug("<<")
        self.close_window()
    
    def on_cancel_clicked(self,widget):
        self.close_window()

    def close_window(self, widget=None):
        self.newrecord.hide()
        #self.newrecord = None
        self.quit()

    def on_calendar_clicked(self,widget):
        calendardialog = WindowCalendar(self.data_path,self, date=self.rcd_date.get_text())
        calendardialog.run()

    def setDate(self,date):
        self.rcd_date.set_text(date)

    def setTime(self,timeInSeconds):
        time_in_hour = int(timeInSeconds)/3600.0
        hour = int(time_in_hour)
        min = int((time_in_hour-hour)*60)
        sec = (((time_in_hour-hour)*60)-min)*60
        self.rcd_hour.set_value(hour)
        self.rcd_min.set_value(min)
        self.rcd_second.set_value(sec)

    def setValue(self,var_name,value, format="%0.2f"):
        var = getattr(self,var_name)
        try:
            valueString = format % value
            var.set_text(valueString)
        except Exception as e:
            print var_name, value, e
            pass
    
    def setValuesFromActivity(self, activity):
        logging.debug(">>")
        self.mode = "editrecord"
        if activity is None:
            logging.debug("activity is None")
            logging.debug("<<")
            return
        self.id_record = activity.id
        (h, m, s) = activity.time_tuple
        self.rcd_hour.set_value(h)
        self.rcd_min.set_value(m)
        self.rcd_second.set_value(s)
        self.rcd_date.set_text(str(activity.date))
        
        #self.rcd_distance.set_text("%.2f"%activity.distance)
        #myset_text(self.rcd_distance, 'distance', activity.distance, us=self.us, round=2)
        if activity.distance is not None:
            self.rcd_distance.set_text("%.2f" %self.uc.distance(activity.distance))        
        else:
            self.rcd_distance.set_text("")
        self.rcd_average.set_text("%.2f" %self.uc.speed(activity.average))
        self.rcd_calories.set_text("%s"%activity.calories)
        self.rcd_beats.set_text("%s"%activity.beats)
        self.rcd_upositive.set_text("%.2f" %self.uc.height(activity.upositive))
        self.rcd_unegative.set_text("%.2f" %self.uc.height(activity.unegative))
        self.rcd_maxvel.set_text("%.2f" %self.uc.speed(activity.maxspeed))
        self.rcd_maxpace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.maxpace),True)) # value coming from DB
        self.rcd_pace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.pace),True)) # value coming from DB
        self.rcd_maxbeats.set_text("%s"%activity.maxbeats)
        self.rcd_title.set_text(activity.title)
        
        if activity.starttime is not None:
            self.rcd_starttime.set_text("%s" % activity.starttime)
        sportPosition = self.getSportPosition(activity.sport_id)
        self.rcd_sport.set_active(sportPosition) 
        buffer = self.rcd_comments.get_buffer()
        start,end = buffer.get_bounds()
        buffer.set_text(activity.comments)
        if activity.gpx_file is not None:
            self.rcd_gpxfile.set_text(activity.gpx_file)
            self.frameGeneral.set_sensitive(0)      #Currently record values not changed if a GPX file is present
            self.frameVelocity.set_sensitive(0)     #Greying out options to indicate this to user
            self.framePace.set_sensitive(0)
            self.frameElevation.set_sensitive(0)
            self.frameBeats.set_sensitive(0)
        logging.debug("<<")
        
    def setValues(self,values):
        #(24, u'2009-12-26', 4, 23.48, u'9979', 0.0, 8.4716666232200009, 2210, u'', None, u'', 573.0, 562.0, 11.802745244400001, 5.0499999999999998, 7.04, 0.0, u'2009-12-25T19:41:48Z', u'2009-12-26 08:41:48+13:00')
        #(50, u'2006-10-13', 1, 25.0, u'5625', 0.0, 16.0, 0, u'', gpsfile, title,upositive,unegative,maxspeed|maxpace|pace|maxbeats
        print "windowrecord setValues called"
        self.mode = "editrecord"
        self.id_record = values[0]
        self.setTime(values[4])
        self.rcd_date.set_text(str(values[1]))
        self.setValue("rcd_distance",values[3])
        self.setValue("rcd_average",values[6])
        self.setValue("rcd_calories",values[7], "%0.0f")
        self.setValue("rcd_beats",values[5], "%0.0f")
        self.setValue("rcd_upositive",values[11])
        self.setValue("rcd_unegative",values[12])
        self.setValue("rcd_maxvel",values[13])
        self.setValue("rcd_maxpace",values[14])
        self.setValue("rcd_pace",values[15])
        self.setValue("rcd_maxbeats",values[16], "%0.0f")
        self.rcd_title.set_text("%s"%values[10])
        
        local_time = values[18]
        if local_time is not None:
            dateTime = dateutil.parser.parse(local_time)
            sTime = dateTime.strftime("%X")
            self.rcd_starttime.set_text("%s" % sTime)
        sportID = values[2]
        sportPosition = self.getSportPosition(sportID)
        self.rcd_sport.set_active(sportPosition) 
        buffer = self.rcd_comments.get_buffer()
        start,end = buffer.get_bounds()
        buffer.set_text(values[8])

    def getSportPosition(self, sportID):
        """
            Function to determine the position in the sport array for a given sport ID
            Needed as once sports are deleted there are gaps in the list...
        """
        count = 0
        for key, value in self.listSport.iteritems():
            if key == sportID: 
                return count
            count +=1
        return 0
        
    def getSportPositionByName(self, sport):
        """
            Function to determine the position in the sport array for a given sport 
            Needed as once sports are deleted there are gaps in the list...
        """
        count = 0
        for key, value in self.listSport.iteritems():
            if value == sport: 
                return count
            count +=1
        return None
    
    def on_calctime_clicked(self,widget):
        logging.debug(">>")
        try:
            distance = float(self.rcd_distance.get_text()) # distance is mandatory!
            # we need either pace or speed
            try:
                average = float(self.rcd_average.get_text())
                time_in_hour = distance/average
                logging.debug("Distance: %0.3f km (mi) | Speed: %0.2f -> Time: %.f hours  " %(distance,average,time_in_hour)) 
                pace = self.parent.pace_from_float(60/average)
                logging.debug("Setting pace: %s" %pace)
                self.rcd_pace.set_text(pace)
            except:
                pace_dec = self.parent.pace_to_float(self.rcd_pace.get_text())
                time_in_hour = pace_dec*distance/60.0
                logging.debug("Distance: %0.3f km (mi) | Pace_dec: %0.2f -> Time: %.f hours" %(distance,pace_dec,time_in_hour))
                speed = distance/time_in_hour
                logging.debug("Setting average speed: %0.2f" %speed)
                self.rcd_average.set_text("%0.2f" %speed)
            self.set_recordtime(time_in_hour)
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
            pass
        logging.debug("<<")

    def update_activity_data(self, row, gpx_file, sport):
        logging.debug(">>")
        self.activity_data[row]["rcd_comments"] = ""
        gpx_summary, laps = self.parent.summaryFromGPX(gpx_file, (sport,""))
        local_time = gpx_summary['date_time_local']
        start_date = local_time.strftime("%Y-%m-%d")
        start_time = local_time.strftime("%H:%M:%S")
        self.activity_data[row]["rcd_date"] = start_date
        self.activity_data[row]["rcd_starttime"] = start_time
        self.activity_data[row]["date_time_local"] = gpx_summary['date_time_local']
        self.activity_data[row]["date_time_utc"] = gpx_summary['date_time_utc']
        self.activity_data[row]["rcd_time"] = gpx_summary["rcd_time"]
        self.activity_data[row]["rcd_distance"] = gpx_summary["rcd_distance"]
        self.activity_data[row]["rcd_average"] = gpx_summary["rcd_average"]
        self.activity_data[row]["rcd_calories"] = gpx_summary["rcd_calories"]
        self.activity_data[row]["rcd_beats"] = gpx_summary["rcd_beats"]
        self.activity_data[row]["rcd_upositive"] = gpx_summary["rcd_upositive"]
        self.activity_data[row]["rcd_unegative"] = gpx_summary["rcd_unegative"]
        self.activity_data[row]["rcd_maxvel"] = gpx_summary["rcd_maxvel"]
        self.activity_data[row]["rcd_maxpace"] = gpx_summary["rcd_maxpace"]
        self.activity_data[row]["rcd_pace"] = gpx_summary["rcd_pace"]
        self.activity_data[row]["rcd_maxbeats"] = gpx_summary["rcd_maxbeats"]
        self.activity_data[row]["rcd_title"] = ""
        self.activity_data[row]["laps"] = laps
        self.activity_data[row]["complete"] = True
        logging.debug("<<")

        
    def show_treeviewEntries_row(self, row):
        '''
        Show details of treeview entry
        TODO need to maintain any changes and display those....
        '''
        logging.debug(">>")
        self.active_row = row
        #Get details from stored data
        #set sport
        sport = self.activity_data[row]["rcd_sport"]
        sportPosition = self.getSportPositionByName(sport)
        if sportPosition is not None:
            self.rcd_sport.set_active(sportPosition)
        #Set gpx file name
        gpx_file = self.activity_data[row]["rcd_gpxfile"]
        self.setValue("rcd_gpxfile", gpx_file, "%s")
        #set duration
        time = time2second(self.activity_data[row]["rcd_time"])      #TODO Fix to use timeinseconds!!
        self.setTime(time)                                                  #TODO Fix to use timeinseconds!!
        #Set distance
        self.setValue("rcd_distance",self.activity_data[row]["rcd_distance"], "%s")
        #Set comments
        buffer = self.rcd_comments.get_buffer()
        start,end = buffer.get_bounds()
        if "rcd_comments" not in self.activity_data[row]:
            self.activity_data[row]["rcd_comments"] = ""
        buffer.set_text(self.activity_data[row]["rcd_comments"])
        while gtk.events_pending(): # This allows the GUI to update 
            gtk.main_iteration()    # before completion of this entire action
        if self.activity_data[row]["complete"] is False:
            #Haven't processed GPX file yet
            #Blank values not yet known
            self.setValue("rcd_date", "", "%s")
            self.setValue("rcd_starttime", "", "%s")
            self.setValue("rcd_average", "", "%s")
            self.setValue("rcd_calories","", "%s")
            self.setValue("rcd_beats", "", "%s")
            self.setValue("rcd_upositive", "", "%s")
            self.setValue("rcd_unegative", "", "%s")
            self.setValue("rcd_maxvel", "", "%s")
            self.rcd_maxpace.set_text("")
            self.rcd_pace.set_text("")
            self.setValue("rcd_maxbeats", "", "%s")
            while gtk.events_pending(): # This allows the GUI to update 
                gtk.main_iteration()    # before completion of this entire action
            #Get some info from gpx file
            self.update_activity_data(row, gpx_file, sport)
        self.setValue("rcd_distance",self.activity_data[row]["rcd_distance"], "%s") 
        time = time2second(self.activity_data[row]["rcd_time"])
        self.setTime(time)  
        self.setValue("rcd_date", self.activity_data[row]["rcd_date"], "%s")
        self.setValue("rcd_starttime", self.activity_data[row]["rcd_starttime"], "%s")
        self.setValue("rcd_average",self.activity_data[row]["rcd_average"])
        self.setValue("rcd_calories",self.activity_data[row]["rcd_calories"], "%s")
        self.setValue("rcd_beats",self.activity_data[row]["rcd_beats"], "%s")
        self.setValue("rcd_upositive",self.activity_data[row]["rcd_upositive"], "%s")
        self.setValue("rcd_unegative",self.activity_data[row]["rcd_unegative"], "%s")
        self.setValue("rcd_maxvel",self.activity_data[row]["rcd_maxvel"])
        self.rcd_maxpace.set_text(self.activity_data[row]["rcd_maxpace"])
        self.rcd_pace.set_text(self.activity_data[row]["rcd_pace"])
        self.setValue("rcd_maxbeats",self.activity_data[row]["rcd_maxbeats"], "%s")
        self.rcd_title.set_text(self.activity_data[row]["rcd_title"])
        logging.debug("<<")
        
        
    def on_rcd_title_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            self.activity_data[self.active_row]["rcd_title"] = self.rcd_title.get_text()
            
    def on_rcd_sport_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            sport = self.rcd_sport.get_active_text()
            #Update sport in data store
            self.activity_data[self.active_row]["rcd_sport"] = sport
            #Update sport in treeview
            self.store[self.active_row][4] = sport
            
    def on_rcd_distance_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            distance = self.rcd_distance.get_text()
            #Update distance in data store
            self.activity_data[self.active_row]["rcd_distance"] = distance
            #Update distance in treeview
            self.store[self.active_row][2] = distance
            
    def on_rcd_duration_value_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            hour = self.rcd_hour.get_value()
            min = self.rcd_min.get_value()
            sec = self.rcd_second.get_value()
            #print hour, min, sec
            #Update duration in data store
            self.activity_data[self.active_row]["rcd_time"] = (hour, min, sec)
            #Update duration in treeview
            self.store[self.active_row][3] = "%d:%.2d:%.2d" % (int(hour), int(min), int(sec))
            
    def on_rcd_date_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            #Update date in data store
            self.activity_data[self.active_row]["rcd_date"] = self.rcd_date.get_text()
            
    def on_rcd_starttime_changed(self, widget):
        if self.mode == "multiple_activities" and self.active_row is not None:
            #Update start time in data store
            self.activity_data[self.active_row]["rcd_starttime"] = self.rcd_starttime.get_text()
            
    def on_treeviewEntries_row_activated(self, treeview, event):
        '''
         Callback to display details of different activity
        '''
        #Check for edited data in previous row
        if self.active_row is not None:
            #Check for edited data in comments
            buffer = self.rcd_comments.get_buffer()
            start,end = buffer.get_bounds()
            comments = buffer.get_text(start,end, True).replace("\"","'")
            self.activity_data[self.active_row]["rcd_comments"] = comments
            #Advanced tab items
            self.activity_data[self.active_row]["rcd_maxpace"] = self.rcd_maxpace.get_text()
            self.activity_data[self.active_row]["rcd_pace"] = self.rcd_pace.get_text()
            self.activity_data[self.active_row]["rcd_upositive"] = self.rcd_upositive.get_text()
            self.activity_data[self.active_row]["rcd_unegative"] = self.rcd_unegative.get_text()
            self.activity_data[self.active_row]["rcd_maxbeats"] = self.rcd_maxbeats.get_text()
            self.activity_data[self.active_row]["rcd_beats"] = self.rcd_beats.get_text()
            self.activity_data[self.active_row]["rcd_calories"] = self.rcd_calories.get_text()
        #Get row that was selected
        x = int(event.x)
        y = int(event.y)
        time = event.time
        pthinfo = treeview.get_path_at_pos(x, y)
        if pthinfo is not None:
            path, col, cellx, celly = pthinfo
            treeview.grab_focus()
            treeview.set_cursor(path, col, 0)
            while gtk.events_pending(): # This allows the GUI to update 
                gtk.main_iteration()    # before completion of this entire action
            self.show_treeviewEntries_row(path[0])

    def on_calcavs_clicked(self,widget):
        logging.debug(">>")
        hour = self.rcd_hour.get_value_as_int()
        min = self.rcd_min.get_value_as_int()
        sec = self.rcd_second.get_value_as_int()
        time = sec + (min*60) + (hour*3600)
        if time<1:
            logging.debug("Seems no time value (%s) has been entered, nothing to calculate." %time)
            return False
        distance = float(self.rcd_distance.get_text())
        if distance<1:
            logging.debug("Seems no distance value (%s) has been entered, nothing to calculate." %distance)
            return False
        logging.debug("Time: %d seconds | Distance: %0.2f km (mi)" %(time,distance))
        # Average speed        
        average_speed = distance*3600.0/time
        logging.debug("Average speed: %0.2f" %average_speed)
        self.rcd_average.set_text("%0.2f" %average_speed)
        # Average pace 
        dec_pace = 60/average_speed
        #Transform pace to mm:ss
        pace = self.parent.pace_from_float(dec_pace)
        logging.debug("Average pace: %s" %pace)
        self.rcd_pace.set_text(pace)
        logging.debug("<<")

    def on_calccalories_clicked(self,widget):
        sport = self.rcd_sport.get_active_text()
        hour = self.rcd_hour.get_value_as_int()
        min = self.rcd_min.get_value_as_int()
        sec = self.rcd_second.get_value_as_int()
        hour += float(min)/60.0 + float(sec)/(60.0*60.0)
        weight = self.pytrainer_main.profile.getValue("pytraining","prf_weight")
        try:
            weight = float(weight)
        except:
            weight = 0.0
        try:
            met = float(self.parent.getSportMet(sport))
        except:
            met = None
        try:
            extraweight = self.parent.getSportWeight(sport)
            extraweight = float(extraweight)
        except:
            extraweight = 0.0
        if met is not None:
            calories = met*(weight+extraweight)*hour
            self.rcd_calories.set_text(str(calories))

    def on_calcdistance_clicked(self,widget):
        logging.debug(">>")
        try:
            hour = self.rcd_hour.get_value_as_int()
            min = self.rcd_min.get_value_as_int()
            sec = self.rcd_second.get_value_as_int()
            time = sec + (min*60) + (hour*3600)
            time_in_hour = time/3600.0
            # we need either pace or speed
            try:
                average = float(self.rcd_average.get_text())
                distance = average*time_in_hour
                logging.debug("Time: %d seconds | Speed: %0.2f -> Distance: %0.3f km (mi)" %(time,average,distance)) 
                pace = self.parent.pace_from_float(60/average)
                logging.debug("Setting pace: %s" %pace)
                self.rcd_pace.set_text(pace)
            except:
                pace_dec = self.parent.pace_to_float(self.rcd_pace.get_text())
                distance = time/(60.0*pace_dec)
                logging.debug("Time: %d seconds | Pace_dec: %0.2f -> Distance: %0.3f km (mi)" %(time,pace_dec,distance))
                speed = distance/time_in_hour
                logging.debug("Setting average speed: %0.2f" %speed)
                self.rcd_average.set_text("%0.2f" %speed)
            self.set_distance(distance) 
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
            pass
        logging.debug("<<")
    
    def set_distance(self,distance):
        self.rcd_distance.set_text("%0.2f" %distance)
        #myset_text(rcd_distance, 'distance', distance, us=self.us, round=2)
        
    def set_maxspeed(self,vel):
        self.rcd_maxvel.set_text("%0.2f" %vel)
            
    def set_maxhr(self,hr):
        self.rcd_maxbeats.set_text("%0.2f" %hr)
            
    def set_recordtime (self,time_in_hour):
        hour = int(time_in_hour)
        min = int((time_in_hour-hour)*60)
        sec = (((time_in_hour-hour)*60)-min)*60
        self.rcd_hour.set_value(hour)
        self.rcd_min.set_value(min)
        self.rcd_second.set_value(sec)

    def on_selectfile_clicked(self,widget):
        logging.debug(">>")
        from pytrainer.gui.dialogs import fileChooserDialog
        selectedFile = fileChooserDialog(title="Choose a Google Earth file (.kml) to import", multiple=False).getFiles()
        if selectedFile is not None:
            self.rcd_gpxfile.set_text(selectedFile[0])
        logging.debug("<<")

    def set_gpxfile(self):
        logging.debug(">>")
        logging.debug("<<")

    def on_calculatevalues_clicked(self,widget):
        gpxfile = self.rcd_gpxfile.get_text()
        if os.path.isfile(gpxfile):
            self.frameGeneral.set_sensitive(0)
            self.frameVelocity.set_sensitive(0) 
            self.parent.actualize_fromgpx(gpxfile)




#!/usr/bin/env python
import os, stat, sys
import logging

import random, re, urllib2, zipfile
from math import floor, ceil
from cStringIO import StringIO
from osgeo import gdal, gdalnumeric

from pytrainer.lib.srtmtiff import SrtmTiff
import srtmdownload

class SrtmLayer(object):
    """
    Provides an interface to SRTM elevation data stored in GeoTIFF files.
    Files are automaticly downloaded from mirror server and cached.
    
    Sample usage:
    
        >>> lat = 52.25
        >>> lon = 16.75
        >>> srtm = SrtmLayer()
        >>> ele = srtm.get_elevation(lat, lon)
        >>> round(ele, 4)
        63.9979
        
    """
    _cache = {}
      
    def get_srtm_filename(self, lat, lon):
        """
        Filename of GeoTIFF file containing data with given coordinates.
        """
        colmin = floor((6000 * (180 + lon)) / 5)
        rowmin = floor((6000 * (60 - lat)) / 5)
    
        ilon = ceil(colmin / 6000.0)
        ilat = ceil(rowmin / 6000.0)
        
        return 'srtm_%02d_%02d.tif' % (ilon, ilat)
        
    def get_elevation(self, lat, lon):
        """
        Returns the elevation in metres of point (lat, lon).
        """
        srtm_filename = self.get_srtm_filename(lat, lon)
        if srtm_filename not in self._cache:
            srtm_path = os.path.join(os.path.expanduser('~/.pytrainer/SRTM_data'), srtm_filename)
            if not os.path.isfile(srtm_path):
                result = srtmdownload.download( srtm_filename[:-4] )
                if not result:
                    return False
            else:
                print "File already downloaded (%s)" % srtm_filename
                            
            self._cache[srtm_filename] = SrtmTiff(srtm_path)
        
        srtm = self._cache[srtm_filename]
        return srtm.get_elevation(lat, lon)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import datetime
from timegraph import TimeGraph

class WeekGraph(TimeGraph):

	value_params = [
		(None, _("Distance (km)"),_("Daily Distance"), None),
		(None, _("Time (hours)"), _("Daily Time"), None),
		(None, _("Average Heart Rate (bpm)"), _("Daily Average Heart Rate"), None),
		(None, _("Average Speed (km/h)"), _("Daily Average Speed"), None),
		(None, _("Calories"), _("Daily Calories"), None),
	]

	def __init__(self, sports, vbox = None, window = None, combovalue = None, combovalue2 = None, main = None):
		TimeGraph.__init__(self, sports, vbox=vbox, window=window, main=main)
		self.combovalue = combovalue
		self.combovalue2 = combovalue2
		self.KEY_FORMAT = "%a"

	def drawgraph(self,values, date_ini):
		TimeGraph.drawgraph(self, values, x_func=lambda x: getDays(date_ini))

def getDays(date_ini):
	#TODO look at using calendar.day_abbr for this
	return [unicode((date_ini+datetime.timedelta(x)).strftime("%a")) for x in xrange(0,7)]




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import traceback
import subprocess
from lxml import etree
from pytrainer.lib.date import getDateTime
from pytrainer.lib.xmlUtils import XMLParser

class garminfit():
    '''First approach to parse Garmin FIT files is to use perl scripts (http://pub.ks-and-ks.ne.jp/cycling/fit2tcx.shtml) from Kiyokazu SUTO (suto@ks-and-ks.ne.jp) to convert first to TCXv2 and then to GPX+ format.
        Another step would be to use other parsers like GPSBabel (http://www.gpsbabel.org/htmldoc-1.4.3/fmt_garmin_fit.html) and python-fitparse (https://github.com/dtcooper/python-fitparse)
     '''
    def __init__(self, parent = None, data_path = None):
        self.parent = parent
        if parent is not None:
        	self.pytrainer_main = parent.parent
        	self.tmpdir = self.pytrainer_main.profile.tmpdir
        if data_path is not None:
        	self.main_data_path = data_path
        	self.data_path = os.path.dirname(__file__)
        self.xmldoc = None
        self.activitiesSummary = []
        self.activities = []

    def getXmldoc(self):
        ''' Function to return parsed xmlfile '''
        return self.xmldoc

    def getFileType(self):
        return "Garmin Flexible and Interoperable data Transfer (FIT)"

    def getActivitiesSummary(self):
        return self.activitiesSummary

    def getDetails(self, activity, startTime):
        logging.debug(">>")
        distance = 0
        duration = 0
        laps = activity.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Lap")
        if laps:
            for lap in laps:
                lap_duration = float(lap.findtext(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}TotalTimeSeconds"))
                lap_distance = float(lap.findtext(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}DistanceMeters"))
                logging.debug("Lap distance (m): %f | duration (s): %f" % (lap_distance, lap_duration))
                distance += lap_distance
                duration += lap_duration
                hours = int(duration)//3600
                minutes = (int(duration)/60)%60
                seconds = int(duration)%60
                duration_hhmmss = "%02d:%02d:%02d" % (hours, minutes, seconds)       
            logging.debug("Activity distance (m): %f | duration (hh:mm:ss - s): %s - %f" % (distance, duration_hhmmss, duration))
        else:
            points = activity.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Trackpoint")
            while True:
                lastPoint = points[-1]
                try:
                    distance = lastPoint.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}DistanceMeters")
                    if distance is None:
                        points = points[:-1]
                        continue
                    time = lastPoint.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Time")
                    distance = distance.text
                    time = time.text
                    break
                except:
                    #Try again without the last point (i.e work from end until find time and distance)
                    points = points[:-1]
                    continue
            duration_hhmmss = getDateTime(time)[0]-startTime[0]
            logging.debug("Activity distance (m): %s | duration (hh:mm:ss): %s" % (distance, duration_hhmmss))
        logging.debug("<<")
        return float(distance), duration_hhmmss

    def fromFIT2TCXv2 (self, filename):
        ''' Reads binary fit file and returns xml string in TCXv2 format'''
        logging.debug(">>")
        result = False
        try:
            pipe = subprocess.Popen(["perl", self.main_data_path+"plugins/garmin-fit/bin/fit2tcx", filename], stdout=subprocess.PIPE)
            result = pipe.stdout.read()
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
        logging.debug("<<")
        return result

    def validate(self, xmldoc, schema):
        xmlschema_doc = etree.parse(self.main_data_path + schema)
        xmlschema = etree.XMLSchema(xmlschema_doc)
        return xmlschema.validate(xmldoc)

    def buildActivitiesSummary(self):
        self.activities = self.getActivities()
        for activity in self.activities:
            startTime = getDateTime(self.getStartTimeFromActivity(activity))
            inDatabase = self.inDatabase(startTime)
            sport = self.getSport(activity)
            distance, duration  = self.getDetails(activity, startTime)
            distance = distance / 1000.0
            self.activitiesSummary.append((self.activities.index(activity),
                                                                 inDatabase, 
                                                                 startTime[1].strftime("%Y-%m-%dT%H:%M:%S"), 
                                                                 "%0.2f" % distance , 
                                                                 str(duration), 
                                                                 sport,
                                                                 ))

    def testFile(self, filename):
        logging.debug('>>')
        logging.debug("Testing %s" %filename)
        try:
            # Original file is in fit format, so first it will be translated into tcxv2 and then parsed
            xmldoc = etree.fromstring(self.fromFIT2TCXv2(filename))
            valid_xml = self.validate(xmldoc, "schemas/GarminTrainingCenterDatabase_v2.xsd")
            if (valid_xml):
                logging.debug("FIT file %s converted to valid TCXv2" %filename)
                self.xmldoc = xmldoc
                self.buildActivitiesSummary()
                return True
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
            return False 
        return False

    def getActivities(self):
        '''Function to return all activities in Garmin training center version 2 file
        '''
        activities = self.xmldoc.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Activity")
        return activities

    def inDatabase(self, startTime):
        #comparing date and start time (sport may have been changed in DB after import)
        logging.debug('>>')
        result = False
        if startTime is not None:
            logging.info("Checking if activity from %s exists in db" % startTime[0])
            time = startTime[0].strftime("%Y-%m-%dT%H:%M:%SZ")
            if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
                result = True
        else:
            logging.info("No start time provided, nothing to check")
        logging.debug('<<')
        return result

    def getSport(self, activity):
        try:
            sport = activity.get("Sport")
        except:
            sport = "import"
        return sport

    def getStartTimeFromActivity(self, activity):
        timeElement = activity.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Id")
        if timeElement is None:
            return None
        else:
            return timeElement.text

    def getGPXFile(self, ID, file_id):
        """
               Generate GPX file based on activity ID

               Returns (sport, GPX filename)
        """
        sport = None
        gpxFile = None
        activityID = int(ID)
        activitiesCount = len(self.activities)
        if activitiesCount > 0 and activityID < activitiesCount:
            gpxFile = "%s/garmin-tcxv2-%s-%d.gpx" % (self.tmpdir, file_id, activityID)
            activity = self.activities[int(activityID)]
            sport = self.getSport(activity)
            self.createGPXfile(gpxFile, activity)
        return sport, gpxFile  

    def createGPXfile(self, gpxfile, activity):
        """ Function to transform a Garmin Training Center v2 Track to a valid GPX+ file
        """
        xslt_doc = etree.parse(self.data_path+"/translate_garmintcxv2.xsl")
        transform = etree.XSLT(xslt_doc)
        xml_doc = activity
        result_tree = transform(xml_doc)
        result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

#Copyright (C) DJ dj@namurlug.org   http://blog.dedj.be

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import xmlrpclib
import SOAPpy
import os
from pytrainer.lib.date import second2time
from pytrainer.lib.soapUtils import *

from optparse import OptionParser


class Main:
	def __init__(self,options):
		# on definit les parametres de connection au blog
		self.xmlrpcserver = options.xmlrpcserver
		self.blogid = options.blogid
		self.user = options.bloguser
		self.password = options.blogpass
		self.category_nbr = options.blogcategory
		#if you want accept comment, replace False by True
		self.comment = False
		#or ping
		self.ping = False
		
		
		self.category = [{'isPrimary': True, "categoryId" : self.category_nbr}]
		
		self.error = ""
		self.log =""
		self.idrecord = options.idrecord
		self.webserviceserver = SOAPpy.SOAPProxy("http://localhost:8081/")

		#we try the connection to the xml/rpc server
		try :
			self.connect = xmlrpclib.Server(self.xmlrpcserver)
			self.error = False
		except : 
			print "can't connect the server"
			
	def loadRecordInfo(self):
      		record = self.webserviceserver.getRecordInfo(self.idrecord)
		self.sport = record["sport"]
                self.date = record["date"]
                self.distance = record["distance"]
                self.time = second2time(float(record["time"]))
                self.heure = self.time[0]
                self.minute = self.time[1] 
                self.seconde = self.time[2]
                self.beats = record["beats"]
                self.comments = record["comments"]
                self.average = record["average"]
                self.calories = record["calories"]
                self.title = record["title"]
                self.upositive = record["upositive"]
                self.unegative = record["unegative"]

	def construction(self):
		#this methode create the content post content, the section between the '''   are on html  thake wath you want
		#self.date, self.distance, self.time, self.beats, self.comments, self.average, self.calories, self.title, self.title, self.upositive, self.unegative, self.sport
		description_table = '''
			<p>%s %s</p>
			<p>With a average of %s km/h</p><br />
			<p>I have lost %s kcalories and my heart have %s pulsations (on average)</p>
			''' %(self.sport, self.title,self.average, self.calories, self.beats)
		return description_table

	def chapeau(self):
		chapeau_table = '''
			<p>A %s score on a distance of %s km on %sh %sm %s </p><br />
			''' %(self.sport,self.distance,self.heure, self.minute, self.seconde)
		return chapeau_table

	def run(self):
		#we load all info for the record
		self.loadRecordInfo()
		blog_desc = self.construction()
		blog_chap = self.chapeau()

		if self.error == False:
			#post_description = "Du " + str(self.sport) + " sur une distance de " + str(self.distance) +"km " + " en " + str(self.time) + " ce qui nous fait une moyenne de " + str(self.average)
			server = xmlrpclib.Server(options.xmlrpcserver)
			content = {'title' : self.date, 'description' : blog_desc, 'mt_allow_comments' : self.comment, 'mt_allow_pings' : self.ping,'mt_excerpt' : blog_chap}
			post = server.metaWeblog.newPost(self.blogid, self.user, self.password, content , True)
			#we change the post categories because, i( or it's the false of xml/rpc) can't select the categories before the upload on the blog 
			change_cat = server.mt.setPostCategories(post,self.user,self.password, self.category)
			return "The post has been submited" 
			self.webserviceserver.stop
        				
		else:
			return self.log


parser = OptionParser()
parser.add_option("-d", "--device", dest="device")
parser.add_option("-k", "--xmlrpcserver", dest="xmlrpcserver")
parser.add_option("-u", "--bloguser", dest="bloguser")
parser.add_option("-p", "--blogpass", dest="blogpass")
parser.add_option("-l", "--blogid", dest="blogid")
parser.add_option("-c", "--blogcategory", dest="blogcategory")
parser.add_option("-g", "--gpxfile", dest="gpxfile")
parser.add_option("-i", "--idrecord", dest="idrecord")
(options,args) =  parser.parse_args()

try : 
  x = Main(options)
  x.run()
  print "le score du %s a ete envoye"  % x.date
except xmlrpclib.Fault, f:
  print "ERROR on the server\n   Code %i\n   Reason %s" % (f.faultCode,
                                               f.faultString)



# -*- coding: utf-8 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import sys
import string
import math
import re
import os

import time
from datetime import datetime
import logging
from lxml import etree
from pytrainer.lib.date import getDateTime

# use of namespaces is mandatory if defined
mainNS = string.Template(".//{http://www.topografix.com/GPX/1/1}$tag")
timeTag = mainNS.substitute(tag="time")
trackTag = mainNS.substitute(tag="trk")
trackPointTag = mainNS.substitute(tag="trkpt")
trackPointTagLast = mainNS.substitute(tag="trkpt[last()]")
trackSegTag = mainNS.substitute(tag="trkseg")
elevationTag = mainNS.substitute(tag="ele")
nameTag = mainNS.substitute(tag="name")

gpxdataNS = string.Template(".//{http://www.cluetrust.com/XML/GPXDATA/1/0}$tag")
calorieTag = gpxdataNS.substitute(tag="calories")
hrTag = gpxdataNS.substitute(tag="hr")
cadTag = gpxdataNS.substitute(tag="cadence")
lapTag = gpxdataNS.substitute(tag="lap")
endPointTag = gpxdataNS.substitute(tag="endPoint")
startPointTag = gpxdataNS.substitute(tag="startPoint")
elapsedTimeTag = gpxdataNS.substitute(tag="elapsedTime")
distanceTag = gpxdataNS.substitute(tag="distance")
intensityTag = gpxdataNS.substitute(tag="intensity")
triggerTag = gpxdataNS.substitute(tag="trigger")
summaryTag = gpxdataNS.substitute(tag="summary")

pytrainerNS = string.Template(".//{http://sourceforge.net/projects/pytrainer/GPX/0/1}$tag")
pyt_eleTag = pytrainerNS.substitute(tag="ele")

class Gpx:
    def __init__(self, data_path = None, filename = None, trkname = None):
        logging.debug(">>")
        #print("GPX init-ing")
        global mainNS, timeTag, trackTag, trackPointTag, trackPointTagLast, trackSegTag, elevationTag, nameTag
        self.data_path = data_path
        self.filename = filename
        self.trkname = trkname
        logging.debug(str(data_path)+"|"+str(filename)+"|"+str(trkname))
        self.trkpoints = []
        self.vel_array = []
        self.total_dist = 0
        self.total_dist_trkpts = 0
        self.total_time = 0
        self.total_time_trkpts = 0
        self.upositive = 0
        self.unegative = 0
        self.maxvel = 0
        self.maxhr = 0
        self.hr_average = 0
        self.date = ""
        self.start_time = ""
        self.calories= 0
        self.tree = None
        if filename != None:
            if not os.path.isfile(self.filename):
                return None
            logging.debug("parsing content from "+self.filename)
            self.tree = etree.ElementTree(file=filename).getroot()
            if self.tree.get("version") == "1.0":
                #Got an old GPX file
                logging.debug("Old gpx version")
                mainNS = string.Template(".//{http://www.topografix.com/GPX/1/0}$tag")
                timeTag = mainNS.substitute(tag="time")
                trackTag = mainNS.substitute(tag="trk")
                trackPointTag = mainNS.substitute(tag="trkpt")
                trackPointTagLast = mainNS.substitute(tag="trkpt[last()]")
                trackSegTag = mainNS.substitute(tag="trkseg")
                elevationTag = mainNS.substitute(tag="ele")
                nameTag = mainNS.substitute(tag="name")
            else:
                logging.debug("Importing version %s gpx file" % self.tree.get("version"))
                mainNS = string.Template(".//{http://www.topografix.com/GPX/1/1}$tag")
                timeTag = mainNS.substitute(tag="time")
                trackTag = mainNS.substitute(tag="trk")
                trackPointTag = mainNS.substitute(tag="trkpt")
                trackPointTagLast = mainNS.substitute(tag="trkpt[last()]")
                trackSegTag = mainNS.substitute(tag="trkseg")
                elevationTag = mainNS.substitute(tag="ele")
                nameTag = mainNS.substitute(tag="name")
                intensityTag = mainNS.substitute(tag="intensity")

            logging.debug("getting values...")
            self.Values = self._getValues()
        logging.debug("<<")

    def getMaxValues(self):
        return self.total_dist, self.total_time, self.maxvel, self.maxhr

    def getDate(self):
        return self.date

    def getTrackRoutes(self):
        trks = self.tree.findall(trackTag)
        tracks = []
        retorno = []
        for trk in trks:
            nameResult = trk.find(nameTag)
            if nameResult is not None:
                name = nameResult.text
            else:
                name = _("No Name")
            timeResult = trk.find(timeTag)
            if timeResult is not None:
                time_ = timeResult.text # check timezone
                logging.debug("TimeResult: %s" %time_)
                mk_time = getDateTime(time_)[0]
                time_ = mk_time.strftime("%Y-%m-%d")
            else:
                time_ = _("No Data")
            logging.debug("name: "+name+" | time: "+time_)
            tracks.append((name,time_))
        return tracks

    def getUnevenness(self):
        return self.upositive,self.unegative

    def getTrackList(self):
        return self.Values

    def getHeartRateAverage(self):
        return self.hr_average

    def getCalories(self):
        return self.calories

    def getStart_time(self):
        return self.start_time

    def getLaps(self):
        logging.debug(">>")
        lapInfo = []
        if self.tree is None:
            return lapInfo
        tree  = self.tree
        laps = tree.findall(lapTag)
        logging.debug("Found %d laps" % len(laps))
        if len(laps) == 0:
            #Found no laps, so add single lap with totals
            stLat = self.trkpoints[0]['lat']
            stLon = self.trkpoints[0]['lon']
            lat = self.trkpoints[-1]['lat']
            lon = self.trkpoints[-1]['lon']
            logging.debug("total_time: %s" %self.total_time)
            lapInfo.append((self.total_time, lat, lon, self.calories, self.total_dist*1000, stLat, stLon, "active", self.hr_average, self.maxhr, self.maxvel, "manual"))
        else:
            for lap in laps:
                endPoint = lap.find(endPointTag)
                lat = endPoint.get("lat")
                lon = endPoint.get("lon")
                startPoint = lap.find(startPointTag)
                if startPoint is not None:
                    stLat = startPoint.get("lat")
                    stLon = startPoint.get("lon")
                else:
                    stLat, stLon = "",""
                elapsedTime = lap.findtext(elapsedTimeTag)
                if elapsedTime.count(":") == 2: # got a 0:41:42.14 type elasped time
                    hours, mins, secs = elapsedTime.split(":")
                    elapsedTime = str((int(hours) *3600) + (int(mins) * 60) + float(secs))
                calories = lap.findtext(calorieTag)
                distance = lap.findtext(distanceTag)
                logging.info("Found time: %s, start lat: %s, start lon: %s, end lat: %s end lon: %s cal: %s dist: %s " % (elapsedTime, stLat, stLon, lat, lon, calories, distance))
                intensity = lap.findtext(intensityTag).lower()
                trigger_element = lap.find(triggerTag)
                #logging.debug("Trigger element: %s" % etree.tostring(trigger_element))
                if "kind" in trigger_element.keys():
                    trigger = trigger_element.get("kind").lower()
                else:
                    trigger = trigger_element.text.lower()
                # attribute xpath features are only available in 1.3 (python >= 2.7) - 2012/01/28 - dgranda
                lap_summary = lap.findall(summaryTag)
                num_elements = len(lap_summary)
                summary_dict = {"MaximumSpeed": 0,
                                "AverageHeartRateBpm": None,
                                "MaximumHeartRateBpm": None}
                if num_elements > 0:
                    if num_elements == 1: # old _non compliant_ pytrainer estructure
                        #logging.debug("lap_summary[0]: %s" % etree.tostring(lap_summary[0]))
                        for key in summary_dict.keys():
                            summary_dict[key] = lap_summary[0].findtext(mainNS.substitute(tag=key))
                            logging.debug("%s: %s" % (key, summary_dict[key]))
                    else:
                        for summary_element in lap_summary:
                            summary_dict[summary_element.get('name')] = summary_element.text
                            logging.debug("%s: %s" % (summary_element.get('name'), summary_element.text))
                    max_speed = summary_dict["MaximumSpeed"]
                    avg_hr = summary_dict["AverageHeartRateBpm"]
                    max_hr = summary_dict["MaximumHeartRateBpm"]
                else:
                    logging.info("No summary found")
                logging.info("Intensity: %s | Trigger: %s | Max speed: %s | Average hr: %s | Maximum hr: %s" % (intensity, trigger, max_speed, avg_hr, max_hr))
                lapInfo.append((elapsedTime, lat, lon, calories, distance, stLat, stLon, intensity, avg_hr, max_hr, max_speed, trigger))
        logging.debug("<<")
        return lapInfo

    def _getValues(self):
        '''
        Migrated to eTree XML processing 26 Nov 2009 - jblance
        '''
        logging.debug(">>")
        tree  = self.tree
        # Calories data comes within laps. Maybe more than one, adding them together - dgranda 20100114
        # Distance data comes within laps where present as well - dgranda 20110204
        laps = tree.findall(lapTag)
        if laps is not None and laps != "":
            totalDistance = 0
            totalDuration = 0
            for lap in laps:
                lapCalories = lap.findtext(calorieTag)                
                self.calories += int(lapCalories)
                lapDistance = lap.findtext(distanceTag)
                totalDistance += float(lapDistance)
                lapDuration_tmp = lap.findtext(elapsedTimeTag)
                # When retrieving data from TCX file -> seconds (float)
                # When retrieving data from GPX+ file -> hh:mm:ss
                # EAFP -> http://docs.python.org/glossary.html
                try:
                    lapDuration = float(lapDuration_tmp)
                except ValueError:
                    hour,minu,sec = lapDuration_tmp.split(":")
                    lapDuration = float(sec) + int(minu)*60 + int(hour)*3600
                totalDuration += lapDuration 
                logging.info("Lap distance: %s m | Duration: %s s | Calories: %s kcal" % (lapDistance, lapDuration, lapCalories))
            self.total_dist = float(totalDistance/1000.0) # Returning km
            self.total_time = int(totalDuration) # Returning seconds
            logging.info("Laps - Distance: %.02f km | Duration: %d s | Calories: %s kcal" % (self.total_dist, self.total_time, self.calories))
        else:
            laps = []

        retorno = []
        his_vel = []
        last_lat = None
        last_lon = None
        last_time = None
        total_dist = 0
        dist_elapsed = 0 # distance since the last time found
        total_hr = 0
        tmp_alt = 0
        len_validhrpoints = 0
        trkpoints = tree.findall(trackPointTag)
        if trkpoints is None or len(trkpoints) == 0:
            logging.debug( "No trkpoints found in file")
            return retorno
        logging.debug("%d trkpoints in file" % len(trkpoints))

        date_ = tree.find(timeTag).text
        if date_ is None:
            logging.info("time tag is blank")
            self.date = None
        else:
            mk_time = getDateTime(date_)[1] #Local Date
            self.date = mk_time.strftime("%Y-%m-%d")
            self.start_time = mk_time.strftime("%H:%M:%S")
        waiting_points = []
        logging.debug("date: %s | start_time: %s | mk_time: %s" % (self.date, self.start_time, mk_time))

        for i, trkpoint in enumerate(trkpoints):
            #Get data from trkpoint
            try:
                lat = float(trkpoint.get("lat"))
                lon = float(trkpoint.get("lon"))
            except Exception as e:
                logging.debug(str(e))
                lat = lon = None
            if lat is None or lat == "" or lat == 0 or lon is None or lon == "" or lon == 0:
                logging.debug("lat or lon is blank or zero")
                continue
            #get the heart rate value from the gpx extended format file
            hrResult = trkpoint.find(hrTag)
            if hrResult is not None:
                hr = int(hrResult.text)
                len_validhrpoints += 1
                total_hr += hr          #TODO fix
                if hr>self.maxhr:
                    self.maxhr = hr
            else:
                hr = None
            #get the cadence (if present)
            cadResult = trkpoint.find(cadTag)
            if cadResult is not None:
                cadence = int(cadResult.text)
            else:
                cadence = None

            #get the time
            timeResult = trkpoint.find(timeTag)
            if timeResult is not None:
                date_ = timeResult.text
                mk_time = getDateTime(date_)[0]
                time_ = time.mktime(mk_time.timetuple()) #Convert date to seconds
                if i == 0:
                    time_elapsed = 0
                else:
                    time_elapsed = time_ - self.trkpoints[i-1]['time'] if self.trkpoints[i-1]['time'] is not None else 0
                    if time_elapsed > 10:
                        logging.debug("%d seconds from last trkpt, someone took a break!" % time_elapsed)
                        # Calculating average lapse between trackpoints to add it
                        average_lapse = round(self.total_time_trkpts/i)
                        logging.debug("Adding %d seconds (activity average) as lapse from last point" % average_lapse)
                        self.total_time_trkpts += average_lapse
                    else:
                        self.total_time_trkpts += time_elapsed
            else:
                time_ = None
                time_elapsed = None

            #get the elevation
            eleResult = trkpoint.find(elevationTag)
            rel_alt = 0
            if eleResult is not None:
                try:
                    ele = float(eleResult.text)
                    #Calculate elevation change
                    if i != 0:
                        rel_alt = ele - self.trkpoints[i-1]['ele'] if self.trkpoints[i-1]['ele'] is not None else 0
                except Exception as e:
                    logging.debug(str(e))
                    ele = None
            else:
                ele = None
                
            #Get corrected elevation if it exists
            correctedEleResult = trkpoint.find(pyt_eleTag)
            if correctedEleResult is not None:
                try:
                    corEle = float(correctedEleResult.text)
                    #Calculate elevation change
                except Exception as e:
                    logging.debug(str(e))
                    corEle = None
            else:
                corEle = None

            #Calculate climb or decent amount
            #Allow for some 'jitter' in height here
            JITTER_VALUE = 0  #Elevation changes less than this value are not counted in +-
            if abs(rel_alt) < JITTER_VALUE:
                rel_alt = 0
            if rel_alt > 0:
                self.upositive += rel_alt
            elif rel_alt < 0:
                self.unegative -= rel_alt

            #Calculate distance between two points
            if i == 0: #First point
                dist = None
            else:
                dist = self._distance_between_points(lat1=self.trkpoints[i-1]['lat'], lon1=self.trkpoints[i-1]['lon'], lat2=lat, lon2=lon)

            #Accumulate distances
            if dist is not None:
                dist_elapsed += dist #TODO fix
                self.total_dist_trkpts += dist

            #Calculate speed...
            vel = self._calculate_speed(dist, time_elapsed, smoothing_factor=3)
            if vel>self.maxvel:
                self.maxvel=vel

            #The waiting point stuff....
            #This 'fills in' the data for situations where some times are missing from the GPX file
            if time_ is not None:
                if len(waiting_points) > 0:
                    for ((w_total_dist, w_dist, w_alt, w_total_time, w_lat, w_lon, w_hr, w_cadence, w_corEle)) in waiting_points:
                        w_time = (w_dist/dist_elapsed) * time_elapsed
                        w_vel = w_dist/((w_time)/3600.0)
                        w_total_time += w_time
                        logging.info("Time added: %f" % w_time)
                        retorno.append((w_total_dist, w_alt, w_total_time, w_vel, w_lat, w_lon, w_hr, w_cadence, w_corEle))
                    waiting_points = []
                    dist_elapsed = 0
                else:
                    retorno.append((self.total_dist_trkpts,ele, self.total_time,vel,lat,lon,hr,cadence,corEle))
                    dist_elapsed = 0
            else: # time_ is None
                waiting_points.append((self.total_dist_trkpts, dist_elapsed, ele, self.total_time, lat, lon, hr, cadence, corEle))

            #Add to dict of values to trkpoint list
            self.trkpoints.append({ 'id': i,
                                    'lat':lat,
                                    'lon':lon,
                                    'hr':hr,
                                    'cadence':cadence,
                                    'time':time_,
                                    'time_since_previous': time_elapsed,
                                    'time_elapsed': self.total_time_trkpts,
                                    'ele':ele,
                                    'ele_change': rel_alt,
                                    'distance_from_previous': dist,
                                    'elapsed_distance': self.total_dist_trkpts,
                                    'velocity':vel,
                                    'correctedElevation':corEle,

                                })

        #end of for trkpoint in trkpoints loop

        #Calculate averages etc
        self.hr_average = 0
        if len_validhrpoints > 0:
            self.hr_average = total_hr/len_validhrpoints
        # In case there is no other way to calculate distance, we rely on trackpoints (number of trackpoints is configurable!)
        if self.total_dist is None or self.total_dist == 0:
            self.total_dist = self.total_dist_trkpts
        else:
            dist_diff = 1000*(self.total_dist_trkpts - self.total_dist)
            logging.debug("Distance difference between laps and trkpts calculation: %f m" % dist_diff)
        if self.total_time is None or self.total_time == 0:
            self.total_time = self.total_time_trkpts
        else:
            time_diff = self.total_time_trkpts - self.total_time
            logging.debug("Duration difference between laps and trkpts calculation: %d s" % time_diff)
        logging.info("Values - Distance: %.02f km | Duration: %d s | Calories: %s kcal" % (self.total_dist, self.total_time, self.calories))
        logging.debug("<<")
        return retorno

    def _distance_between_points(self, lat1, lon1, lat2, lon2):
        '''
        Function to calculate the distance between two lat, lon points on the earths surface

        History of this function is unknown....
        -- David "no me mates que esto lo escribi hace anhos"
        -- http://faculty.washington.edu/blewis/ocn499/EXER04.htm equation for the distance between 2 points on a spherical earth
        -- 0.01745329252 = number of radians in a degree
        -- 57.29577951 = 1/0.01745329252 or degrees per radian
        requires
            - start lat and lon as floats
            - finish lat and lon as floats

        returns
            - distance between points in kilometers if successful
            - None if any error situation occurs
        '''
        RADIANS_PER_DEGREE = 0.01745329252
        DEGREES_PER_RADIAN = 57.29577951
        #Check for invalid variables
        for var in (lat1, lon1, lat2, lon2):
            if var is None or var == 0 or var == "":   #TODO Need this?? if (float(lat) < -0.000001) or (float(lat) > 0.0000001):
                return None
            if type(var) is not type(float()):
                return None
        #Convert lat and lon from degrees to radians
        last_lat = lat1*RADIANS_PER_DEGREE
        last_lon = lon1*RADIANS_PER_DEGREE
        tmp_lat = lat2*RADIANS_PER_DEGREE
        tmp_lon = lon2*RADIANS_PER_DEGREE
        #Pasamos la distancia de radianes a metros..  creo / We convert the distance from radians to meters
        try:
            dist=math.acos((math.sin(last_lat)*math.sin(tmp_lat))+(math.cos(last_lat)*math.cos(tmp_lat)*math.cos(tmp_lon-last_lon)))*111.302*DEGREES_PER_RADIAN
        except Exception as e:
            logging.debug(str(e))
            dist=None
        return dist

    def _calculate_speed(self, dist_elapsed, time_elapsed, smoothing_factor=3):
        '''Function to calculate moving average for speed'''

        if dist_elapsed is None or dist_elapsed == 0 or time_elapsed is None or time_elapsed == 0:
            velocity = 0
        else:
            velocity = (dist_elapsed/time_elapsed) * 3600 # 3600 to convert km/sec to km/hour
        self.vel_array.append(velocity)
        if len(self.vel_array)>smoothing_factor:
            self.vel_array.pop(0)
        if len(self.vel_array)<smoothing_factor:
            #Got too few numbers to average
            #Pad with duplicates
            for x in range(len(self.vel_array), smoothing_factor):
                self.vel_array.append(velocity)
        vel = 0
        for v in self.vel_array:
            vel+= v
        vel /= smoothing_factor
        return vel

    def getStartTimeFromGPX(self, gpxFile):
        '''03.05.2008 - dgranda
        Retrieves start time from a given gpx file
        args:
            - gpxFile: path to xml file (gpx format)
        returns: tuple (string with start time as UTC timezone - 2008-03-22T12:17:43Z, datetime of time in local timezone)
        '''
        logging.debug(">>")
        date_time = self.tree.find(timeTag) #returns first instance found
        if date_time is None:
            print "Problems when retrieving start time from "+gpxFile+". Please check data integrity"
            return 0
        dateTime = getDateTime(date_time.text)
        zuluDateTime = dateTime[0].strftime("%Y-%m-%dT%H:%M:%SZ")
        localDateTime = dateTime[1]
        logging.debug(gpxFile+" | "+ date_time.text +" | " + zuluDateTime + " | " + str(localDateTime))
        #print localDateTime
        #return date_time.text
        logging.debug("<<")
        return (zuluDateTime, localDateTime)




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
import sys
import shutil
import logging

import gtk
import httplib2

import wordpresslib     #TODO remove need for this library
from pytrainer.extensions.googlemaps import Googlemaps
import pytrainer.lib.points as Points
from pytrainer.lib.date import Date
from pytrainer.lib.uc import UC

class wordpress:
    def __init__(self, parent = None, pytrainer_main = None, conf_dir = None, options = None):
        #TODO could use some logging
        self.parent = parent
        self.pytrainer_main = pytrainer_main
        self.uc = UC()
        self.options = options
        self.conf_dir = conf_dir
        self.tmpdir = self.pytrainer_main.profile.tmpdir
        self.data_path = self.pytrainer_main.profile.data_path
        self.googlemaps = Googlemaps(data_path=self.data_path, pytrainer_main=pytrainer_main)
        self.wp = None

    def run(self, id, activity=None):
        #Show user something is happening
        msg = _("Posting to Wordpress blog")
        md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, msg)
        md.set_title(_("Wordpress Extension Processing"))
        md.set_modal(True)
        md.show()
        while gtk.events_pending(): # This allows the GUI to update
            gtk.main_iteration()    # before completion of this entire action
        logging.debug("before request posting")
        options = self.options
        self.wordpressurl = options["wordpressurl"]
        self.user = options["wordpressuser"]
        self.password = options["wordpresspass"]
        self.gpxfile = "%s/gpx/%s.gpx" %(self.conf_dir,id)
        self.googlekey = options["googlekey"]
        self.idrecord = id
        self.activity = activity
        self.wordpresscategory = options["wordpresscategory"]
        debug_msg = "%s, %s, %s, %s, %s, %s" % (self.wordpressurl, self.user, self.gpxfile, self.googlekey, self.idrecord, self.wordpresscategory)
        logging.debug(debug_msg)
        try:
            self.wp = wordpresslib.WordPressClient(self.wordpressurl, self.user, self.password) #TODO remove need for wordpresslib??
            self.error = False
        except Exception as e:
            print e
            self.error = True
            self.log = "Url, user or pass are incorrect. Please, check your configuration"
        self.loadRecordInfo()
        if self.title is None or self.title == "":
            self.title = "Untitled Activity"
        blog_table = self.createTable()
        blog_route = self.createRoute()
        blog_body = self.createBody()

        blog_figureHR = self.createFigureHR()
        blog_figureStage = self.createFigureStage()
        blog_foot = self.createFoot()

        self.description = "<![CDATA["+blog_body+blog_table+blog_route+blog_figureHR+blog_figureStage+blog_foot+"]]>"
        xmlstuff = '''<methodCall>
<methodName>metaWeblog.newPost</methodName>
<params>
<param>
<value>
<string>MyBlog</string>
</value>
</param>
<param>
<value>%s</value>
</param>
<param>
<value>
<string>%s</string>
</value>
</param>
<param>
<struct>
<member>
<name>categories</name>
<value>
<array>
<data>
<value>%s</value>
</data>
</array>
</value>
</member>
<member>
<name>description</name>
<value>%s</value>
</member>
<member>
<name>title</name>
<value>%s</value>
</member>
</struct>
</param>
<param>
 <value>
  <boolean>1</boolean>
 </value>
</param>
</params>
</methodCall>
''' % (self.user, self.password, self.wordpresscategory,  self.description, self.title)

        #POST request to Wordpress blog
        h = httplib2.Http()
        res, content = h.request(self.wordpressurl, 'POST', body=xmlstuff)
        logging.debug("after request posting")
        logging.debug("Got response status: %s, reason: %s, content: %s" % (res.status, res.reason, content))
        if res.reason == 'OK':
            res_msg = "Successfully posted to Wordpress."
        else:
            res_msg = "Some error occured\nGot a status %s, reason %s\nContent was: %s" % (res.status, res.reason, content)
        #Close 'Please wait' dialog
        md.destroy()
        #Show the user the result
        md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, res_msg)
        md.set_title(_("Wordpress Extension Upload Complete"))
        md.set_modal(False)
        md.run()
        md.destroy()

    def createRoute(self):
        gpxpath = "%s/gpstrace.gpx.txt" % self.tmpdir
        #htmlpath = "%s/index.html" % self.tmpdir
        kmlpath = "%s/gps.kml.txt" % self.tmpdir
        description_route = ''
        if os.path.isfile(self.gpxfile):
            #copy gpx file to new name
            shutil.copy(self.gpxfile, gpxpath)
            #create the html file
            #htmlpath = googlemaps.drawMap(self.gpxfile,self.googlekey,htmlpath)    #TODO fix to use main googlemaps and remove extensions copy
            htmlpath = self.googlemaps.drawMap(self.activity)
            #create the kml file
            os.system("gpsbabel -t -i gpx -f %s -o kml,points=0,line_color=ff0000ff -F %s" %(self.gpxfile,kmlpath)) #TODO fix to remove gpsbabel

            #gfile = self.wp.newMediaObject(self.gpxfile)
            gfile = self.wp.newMediaObject(gpxpath)
            hfile = self.wp.newMediaObject(htmlpath)
            kfile = self.wp.newMediaObject(kmlpath)

            description_route = '''<strong>Map: </strong> <br/>
            <iframe width='480' height='480' src='%s'> Need frame support </iframe><br/>
            <a href='%s'>Gpx-format</a> <a href='%s'>Kml-format (GoogleEarth)</a><br/><br/>''' %(hfile,gfile,kfile)
        return description_route

    def loadRecordInfo(self):
        #date = Date()
        #record = self.pytrainer_main.record.getrecordInfo(self.idrecord)[0]
        #"sports.name,date,distance,time,beats,comments,average,calories,id_record,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats,date_time_utc,date_time_local",
        self.sport = self.activity.sport_name
        self.date = self.activity.date
        self.distance = self.activity.distance
        self.time = "%d:%02d:%02d" % (self.activity.time_tuple)
        self.beats = self.activity.beats
        self.comments = self.activity.comments
        self.average = self.activity.average
        self.calories = self.activity.calories
        self.title = self.activity.title
        self.upositive = self.activity.upositive
        self.unegative = self.activity.unegative
        self.maxspeed = self.activity.maxspeed
        self.maxpace = self.activity.maxpace
        self.pace = self.activity.pace
        self.maxbeats = self.activity.maxbeats

    def createBody(self):
        if self.comments is None or self.comments == "":
            return ""
        else:
            return '''<b> Description: </b><br/>%s<br/>''' %self.comments

    def createTable(self):
        description_table = '''
            <br/>
            <table border=0>
                <tr>
                    <td><strong>Activity:</strong></td>
                    <td>%s</td>
                    <td><strong>Date:</strong></td>
                    <td>%s</td>
                </tr>
                <tr>
                    <td><strong>Distance (%s):</strong></td>
                    <td>%.2f</td>
                    <td><strong>Time (hh:mm:ss):</strong></td>
                    <td>%s</td>
                </tr>
                <tr>
                    <td><strong>Max speed (%s):</strong></td>
                    <td>%.2f</td>
                    <td><strong>Avg speed (%s):</strong></td>
                    <td>%.2f</td>
                </tr>
                <tr>
                    <td><strong>Max pace (%s):</strong></td>
                    <td>%.2f</td>
                    <td><strong>Avg pace (%s):</strong></td>
                    <td>%.2f</td>
                </tr>
                <tr>
                    <td><strong>Max pulse:</strong></td>
                    <td>%s</td>
                    <td><strong>Avg pulse:</strong></td>
                    <td>%s</td>
                </tr>
                <tr>
                    <td><strong>Acc elevation (+%s):</strong></td>
                    <td>%.2f</td>
                    <td><strong>Acc elevation (-%s):</strong></td>
                    <td>%.2f</td>
                </tr>
            </table>
            ''' %(  self.sport, self.date, self.uc.unit_distance, self.distance, self.time, self.uc.unit_speed, self.maxspeed,
                    self.uc.unit_speed, self.average, self.uc.unit_pace, self.maxpace, self.uc.unit_pace, self.pace,
                    self.maxbeats, self.beats, self.uc.unit_height, self.upositive, self.uc.unit_height, self.unegative)
        return description_table

    def createFigureHR(self):
        hr_fig_path = "%s/hr.png" % self.tmpdir #TODO ensure png exists
        blog_figures = ''
        # If there are no graphs, return empty string.
        if os.path.isfile(hr_fig_path):
            #the graph files are created because the graph tabs are automatically visited (which invokes graph generation)
            hrfile = self.wp.newMediaObject(hr_fig_path)
            blog_figures = '''<br/> <img src='%s' /> ''' %hrfile
        return blog_figures

    def createFigureStage(self):
        stage_fig_path = "/tmp/stage.png"   #TODO fix, correct tmp dir and ensure png exists
        blog_figures = ''
        # If there are no graphs, return empty string.
        if os.path.isfile(stage_fig_path):
            #the graph files are created because the graph tabs are automatically visited (which invokes graph generation)
            stagefile = self.wp.newMediaObject(stage_fig_path)
            blog_figures = '''<br/> <img src='%s' /> ''' %stagefile
        return blog_figures

    def createFoot(self):
        return ''' <br/><center>Powered by <a href='https://github.com/pytrainer/pytrainer'>pytrainer</a></center>'''

    def createTitle(self):
        if self.title==None:
            self.error = True
            self.log = "A Title must be defined. Please, configure the record properly"
        return self.title

    def createCategory(self):
        if self.wordpresscategory==None:
            self.error = True
            self.log = "A wordpress category must be defined. Please, configure the wordpress extension"
        else:
            return ([self.wordpresscategory])



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import gtk

class PopupMenu(gtk.Menu):
    def __init__(self, data_path = None, parent = None):
        super(gtk.Menu, self).__init__()
        self.windowmain = parent
        edit_record = gtk.ImageMenuItem(gtk.STOCK_EDIT)
        edit_record.set_label(_("Edit Record"))
        edit_record.connect("activate", self.on_editrecord_activate)
        self.attach(edit_record, 0, 1, 0, 1)
        show_graph = gtk.ImageMenuItem(gtk.STOCK_FIND)
        show_graph.set_label(_("Show graph in classic view"))
        show_graph.connect("activate", self.on_showclassic_activate)
        self.attach(show_graph, 0, 1, 1, 2)
        self.attach(gtk.SeparatorMenuItem(), 0, 1, 2, 3)
        remove_record = gtk.ImageMenuItem(gtk.STOCK_DELETE)
        remove_record.connect("activate", self.on_remove_activate)
        self.attach(remove_record, 0, 1, 3, 4)
    
    def show(self,id_record,event_button, time, date=None):
        self.id_record = id_record
        self.date = date
        self.iter = iter
        self.show_all()
        self.popup(None, None, None, event_button, time)

    def on_editrecord_activate(self,widget):
        self.windowmain.parent.editRecord(self.id_record, view=self.windowmain.selected_view)

    def on_showclassic_activate(self,widget):
        #Set date in classic view
        if self.date is not None:
            self.windowmain.parent.date.setDate(self.date)
        self.windowmain.classicview_item.set_active(True)
        self.windowmain.notebook.set_current_page(0)
        self.windowmain.recordview.set_current_page(0)
        self.windowmain.parent.refreshRecordGraphView("info", id_record=self.id_record)

    def on_remove_activate(self,widget):
        self.windowmain.parent.removeRecord(self.id_record, view=self.windowmain.selected_view)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os, sys, commands
import StringIO
from lxml import etree

class gpsbabel():
	def __init__(self, parent = None, data_path = None):
		self.parent = parent
		self.pytrainer_main = parent.parent
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.main_data_path = data_path
		self.data_path = os.path.dirname(__file__)

	def getName(self):
		return _("GPSBabel")

	def getVersion(self):
		result = commands.getstatusoutput('gpsbabel -V')
		if result[0] == 0:
			version = result[1].split()
			try:
				return version[2]
			except:
				logging.error("Unexpected result from gpsbabel -V")
				return None
		return None

	def getSourceLocation(self):
		return "http://www.gpsbabel.org/"

	def deviceExists(self):
		try:
			#TODO Check if this is correct???
			outmod = commands.getstatusoutput('/sbin/lsmod | grep garmin_gps')
			if outmod[0]==256:	#there is no garmin_gps module loaded
				return False
			else:
				return True
		except:
			return False

	def isPresent(self):
		if self.getVersion():
			return True
		else:
			return False



# -*- coding: iso-8859-1 -*-

#Copyright (C) Sigurður H. Pálsson shp@itn.is

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging

class Stats:
    def __init__(self, sport_service, parent = None):
        logging.debug('>>')
        self._sport_service = sport_service
        self.pytrainer_main = parent
        logging.debug('<<')

    def refresh(self):
        self.data = self.get_stats()

    def get_stats(self):
        logging.debug('>>')
        data = {
            'sports' : {},
            'total_duration' : 0,
            'total_distance' : 0,
        }
        
        fields = ['maxspeed', 'beats', 'maxbeats', 'duration', 'distance']
        data['fields'] = fields
        
        for f in fields:
            data[f] = 0
        
        results = self.pytrainer_main.ddbb.select_dict("records", ('id_record', 'date', 'sport', 'distance', 'duration', 'maxbeats', 'maxspeed', 'maxpace', 'average','pace','beats'))
        for r in results:
            if r['sport'] is None or r['sport'] is 0:
                # temporary fix to ignore corrupt records caused by GPX import bug
                continue
            if r['sport'] not in data['sports']:
                sport = self._sport_service.get_sport(r['sport'])
                data['sports'][r['sport']] = {'name': sport.name, 'count': 0}
                for f in fields:
                    data['sports'][r['sport']][f] = 0
                    data['sports'][r['sport']]['total_'+f] = 0
            data['sports'][r['sport']]['count'] += 1
            for f in fields:
                data['sports'][r['sport']][f] = max(data['sports'][r['sport']][f], r[f])
                if r[f] is not None:
                    data['sports'][r['sport']]['total_'+f] += r[f]
                    data[f] = max(data[f], r[f])
                else:
                    logging.info('Skipping null values')
                    
            if 'avg_hr' not in data['sports'][r['sport']]:
                data['sports'][r['sport']]['avg_hr'] = [0, 0]
            if r['beats']:
                data['sports'][r['sport']]['avg_hr'][0] += 1
                data['sports'][r['sport']]['avg_hr'][1] += r['beats']
                
            data['total_duration'] += r['duration']
            data['total_distance'] += r['distance']
            
            if not r['maxspeed'] and r['duration']:
                data['sports'][r['sport']]['maxspeed'] = max(data['sports'][r['sport']]['maxspeed'], r['distance'] / r['duration'] * 3600)
            
            if not 'start_date' in data: data['start_date'] = r['date']
            data['start_date'] = min(data['start_date'], r['date'])
            if not 'end_date' in data: data['end_date'] = r['date']
            data['end_date'] = max(data['end_date'], r['date'])
            
        for s in data['sports']:
            if data['sports'][s]['avg_hr'][0]:
                data['sports'][s]['avg_hr'] = int(data['sports'][s]['avg_hr'][1] / data['sports'][s]['avg_hr'][0])
            else:
                data['sports'][s]['avg_hr'] = None
            
        logging.debug('<<')
        return data




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from threading import Thread
import webbrowser

def open_url_in_browser(url):
    """
    Opens a url in the desktop's default browser

    :param url: the url to open
    """

    class BrowserThread(Thread):
        def __init__(self, url):
            Thread.__init__(self)
            self.url = url
        def run(self):
            webbrowser.open(self.url)
    BrowserThread(url).start()




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
import sys
import logging

from lib.xmlUtils import XMLParser

from gui.windowplugins import WindowPlugins

class Plugins:
	def __init__(self, data_path = None, parent = None):
		self.data_path=data_path
		self.pytrainer_main = parent
	
	def getActivePlugins(self):
		retorno = []
		for plugin in self.getPluginsList():
			if self.getPluginInfo(plugin[0])[2] == "1":
				retorno.append(plugin[0])
		return retorno	

	def loadPlugin(self,plugin):
		logging.debug('>>')
		info = XMLParser(plugin+"/conf.xml")
		button = info.getValue("pytrainer-plugin","pluginbutton")
		name = info.getValue("pytrainer-plugin","name")
		logging.info('Loading plugin '+name)
		logging.debug('<<')
		return button,plugin

	def importClass(self, pathPlugin):
		logging.debug('>>')
		info = XMLParser(pathPlugin+"/conf.xml")
	  	#import plugin
		plugin_dir = os.path.realpath(pathPlugin) 
		plugin_filename = info.getValue("pytrainer-plugin","executable")
		plugin_classname = info.getValue("pytrainer-plugin","plugincode")
		logging.debug("Plugin Filename: " + plugin_filename )
		logging.debug("Plugin Classname: " + plugin_classname)
		sys.path.insert(0, plugin_dir)
		module = __import__(plugin_filename)
		pluginMain = getattr(module, plugin_classname)
		logging.debug('<<')
		#Only validate files if enabled at startup 
		if self.pytrainer_main.startup_options.validate:
			validate_inputfiles=True
			print "validating plugin input files enabled"
		else:
			validate_inputfiles=False
		return pluginMain(self, validate_inputfiles) 

	def managePlugins(self):
		pluginsList = self.getPluginsList()
		windowplugins = WindowPlugins(self.data_path, self)
		windowplugins.setList(pluginsList)
		windowplugins.run()

	def getPluginsList(self):
		pluginsdir = self.data_path+"/plugins"
		pluginsList = []
		for plugin in os.listdir(pluginsdir):
			pluginxmlfile = pluginsdir+"/"+plugin+"/conf.xml"
			if os.path.isfile(pluginxmlfile):
				plugininfo = XMLParser(pluginxmlfile)
				name = plugininfo.getValue("pytrainer-plugin","name")
				description = plugininfo.getValue("pytrainer-plugin","description")
				pluginsList.append((pluginsdir+"/"+plugin,name,description))
		pluginsList.sort()
		return pluginsList
	
	def getPluginInfo(self,pathPlugin):
		info = XMLParser(pathPlugin+"/conf.xml")
		name = info.getValue("pytrainer-plugin","name")
		description = info.getValue("pytrainer-plugin","description")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir
		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
			status = 0
		else:
			info = XMLParser(plugindir+"/"+code+"/conf.xml")
			status = info.getValue("pytrainer-plugin","status")
		return name,description,status

	def getPluginConfParams(self,pathPlugin):
		info = XMLParser(pathPlugin+"/conf.xml")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir
		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
			params = info.getAllValues("conf-values")
			params.append(("status","0"))
		else:
			prefs = info.getAllValues("conf-values")
			prefs.append(("status","0"))
			info = XMLParser(plugindir+"/"+code+"/conf.xml")
			params = []
			for pref in prefs:
				params.append((pref[0],info.getValue("pytrainer-plugin",pref[0])))
		return params

	def setPluginConfParams(self,pathPlugin,savedOptions):
		info = XMLParser(pathPlugin+"/conf.xml")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir+"/"+code
		if not os.path.isdir(plugindir):
			os.mkdir(plugindir)
		if not os.path.isfile(plugindir+"/conf.xml"):
			if ("status", "1") not in savedOptions:
				savedOptions.append(("status","0"))
		info = XMLParser(plugindir+"/conf.xml")
		info.createXMLFile("pytrainer-plugin",savedOptions)

	def getCodeConfValue(self,code,value):
		plugindir = self.pytrainer_main.profile.plugindir
		info = XMLParser(plugindir+"/"+code+"/conf.xml")
		return info.getValue("pytrainer-plugin",value)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.util.color import Color, color_from_hex_string
import logging

class Sport(object):
    
    """A type of exercise. For example: "running" or "cycling"."""
    
    def __init__(self):
        self._id = None
        self.name = u""
        self.weight = 0.0
        self.met = None
        self.max_pace = None
        self.color = Color(0x0000ff)
       
    def _get_id(self):
        return self._id
    
    def _set_id(self, id):
        self._id = int(id)
    
    id = property(_get_id, _set_id)
       
    def _get_name(self):
        return self._name
    
    def _set_name(self, name):
        if not isinstance(name, unicode):
            raise TypeError("Name must be unicode string, not {0}.".format(type(name).__name__))
        self._name = name
    
    name = property(_get_name, _set_name)
       
    def _get_weight(self):
        return self._weight
    
    def _set_weight(self, weight):
        weight_float = float(weight)
        if weight_float < 0:
            raise ValueError("Weight must not be negative.")
        self._weight = weight_float
    
    weight = property(_get_weight, _set_weight)
       
    def _get_met(self):
        return self._met
    
    def _set_met(self, met):
        met_float = float(met) if met is not None else None
        if met_float is not None and met_float < 0:
            raise ValueError("MET must not be negative.")
        self._met = met_float
    
    met = property(_get_met, _set_met)
       
    def _get_max_pace(self):
        return self._max_pace
    
    def _set_max_pace(self, max_pace):
        max_pace_int = int(max_pace) if max_pace is not None else None
        if max_pace_int is not None and max_pace_int < 0:
            raise ValueError("Max pace must not be negative.")
        self._max_pace = max_pace_int
    
    max_pace = property(_get_max_pace, _set_max_pace)
    
    def _get_color(self):
        return self._color
    
    def _set_color(self, color):
        if color is None:
            raise ValueError("Color must be valued.")
        self._color = color
        
    color = property(_get_color, _set_color)

class SportServiceException(Exception):
    
    def __init__(self, value):
        self.value = value
    
    def __str__(self):
        return repr(self.value)

_TABLE = "sports"

_ID_COLUMN = "id_sports"

_NAME_COLUMN = "name"

_UPDATE_COLUMNS = _NAME_COLUMN + ",weight,met,max_pace,color"

_SELECT_COLUMNS = _ID_COLUMN + "," + _UPDATE_COLUMNS
    
class SportService(object):
    
    """Provides access to stored sports."""
    
    def __init__(self, ddbb):
        self._ddbb = ddbb
        
    def _create_sport(self, row):
        sport = Sport()
        sport.id = row[0]
        sport.name = unicode(row[1])
        sport.weight = row[2]
        sport.met = row[3]
        sport.max_pace = row[4]
        sport.color =  color_from_hex_string(row[5])
        return sport
    
    def _create_row(self, sport):
        return [sport.name,
                sport.weight,
                sport.met,
                sport.max_pace,
                sport.color.to_hex_string()]
        
    def _create_id_where_clause(self, sport_id):
        return _ID_COLUMN + "=" + str(sport_id)
    
    def _create_name_where_clause(self, sport_name):
        return _NAME_COLUMN + "=\"{0}\"".format(sport_name)
    
    def get_sport(self, sport_id):
        """Get the sport with the specified id.

        If no sport with the given id exists then None is returned."""
        if sport_id is None:
            raise ValueError("Sport id cannot be None")
        resultSet = self._ddbb.select(_TABLE, _SELECT_COLUMNS, self._create_id_where_clause(sport_id))
        if len(resultSet) == 0:
            return None
        else:
            return self._create_sport(resultSet[0])
        
    def get_sport_by_name(self, name):
        """Get the sport with the specified name.

        If no sport with the given name exists then None is returned."""
        if name is None:
            raise ValueError("Sport name cannot be None")
        sport_id = self._get_sport_id_from_name(name)
        if sport_id is None:
            return None
        return self.get_sport(sport_id)
        
    def _get_sport_id_from_name(self, name):
        result_set = self._ddbb.select(_TABLE, _ID_COLUMN, self._create_name_where_clause(name))
        if len(result_set) > 0:
            return result_set[0][0]
        return None
    
    def get_all_sports(self):
        """Get all stored sports."""
        result_set = self._ddbb.select(_TABLE, _SELECT_COLUMNS)
        logging.debug("Retrieved all sports ({0} results).".format(len(result_set)))
        sports = []
        for row in result_set:
            sport = self._create_sport(row)
            sports.append(sport)
        return sports
    
    def store_sport(self, sport):
        """Store a new or update an existing sport.
       
       The stored object is returned."""
        if (sport.id is None):
            sport_id = self._store_new_sport(sport)
        else:
            sport_id = self._update_existing_sport(sport)
        return self.get_sport(sport_id)
    
    def _store_new_sport(self, sport):
        self._assert_unique(sport)
        self._ddbb.insert(_TABLE, _UPDATE_COLUMNS, self._create_row(sport))
        logging.debug("Stored new sport: '{0}'.".format(sport.name))
        return self._get_sport_id_from_name(sport.name)
    
    def _update_existing_sport(self, sport):
        self._assert_exists(sport)
        self._assert_unique(sport)
        self._ddbb.update(_TABLE, _UPDATE_COLUMNS, self._create_row(sport), self._create_id_where_clause(sport.id))
        logging.debug("Updated sport: '{0}'.".format(sport.name))
        return sport.id
        
    def _assert_unique(self, sport):
        id = self._get_sport_id_from_name(sport.name)
        if id is not None and id != sport.id:
            raise SportServiceException("A sport already exists with name '{0}'".format(sport.name))
        logging.debug("Asserted sport name is unique: '{0}'.".format(sport.name))
        
    def _assert_exists(self, sport):
        result_set = self._ddbb.select(_TABLE, _ID_COLUMN, self._create_id_where_clause(sport.id))
        if (result_set == []):
            raise SportServiceException("Sport does not exist with id: '{0}'.".format(sport.id))
        logging.debug("Asserted sport exists with id: '{0}'.".format(sport.id))
        
    def remove_sport(self, sport):
        """Delete a stored sport.
        
        All records associated with the sport will also be deleted."""
        if (sport.id is None):
            raise SportServiceException("Cannot remove sport which has not been stored: '{0}'.".format(sport.name))
        self._assert_exists(sport)
        self._ddbb.delete("records", "sport=" + str(sport.id))
        self._ddbb.delete(_TABLE, self._create_id_where_clause(sport.id))
        logging.debug("Deleted sport: '{0}'.".format(sport.name))



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.lib.fileUtils import fileUtils
import gtk
import logging
import webkit

class MapViewer:
	def __init__(self, data_path = None, pytrainer_main=None, box=None):
		logging.debug(">>")
		self.data_path = data_path
		self.pytrainer_main = pytrainer_main
		if box is None:
			logging.debug("Display box (%s) is None" % ( str(box)))
			return
		self.box = box
		self.wkview = webkit.WebView()
		self.pack_box()
		logging.debug("<<")

	def pack_box(self):
		logging.debug(">>")
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.add(self.wkview)
		self.box.pack_start(scrolled_window, True, True)
		self.box.show_all()
		logging.debug("<<")

	def display_map(self, htmlfile=None):
		logging.debug(">>")
		if htmlfile is None:
			htmlfile = self.createErrorHtml()
		self.wkview.load_uri("file://%s" % (htmlfile))
		#self.box.show_all()
		logging.debug("<<")

	def createErrorHtml(self):
		logging.debug(">>")
		htmlfile = "%s/error.html" % (self.pytrainer_main.profile.tmpdir)
		content = '''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"  xmlns:v="urn:schemas-microsoft-com:vml">
<head>
<body>
No HTML file supplied to display
</body>
</html>
		'''
		file = fileUtils(htmlfile,content)
		file.run()
		logging.debug("<<")
		return htmlfile



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest

from mock import Mock

from pytrainer.environment import Environment

TEST_DIR_NAME = "/test/.pytrainer_test"
DATA_DIR_NAME = "/test/datadir"

class Test(unittest.TestCase):

    def setUp(self):
        self.environment = Environment()
        # Environment is a singleton, make sure to destroy it between tests
        del(Environment.self)
        self.environment = Environment(TEST_DIR_NAME, DATA_DIR_NAME)

    def test_get_conf_dir(self):
        self.assertEquals(TEST_DIR_NAME, self.environment.conf_dir)

    def test_get_data_path(self):
        self.assertEquals(DATA_DIR_NAME, self.environment.data_path)

    def test_environment_singleton(self):
        self.environment = Environment()
        self.assertEquals(TEST_DIR_NAME, self.environment.conf_dir)
        self.assertEquals(DATA_DIR_NAME, self.environment.data_path)
        
    def test_get_conf_file(self):
        self.assertEquals(TEST_DIR_NAME + "/conf.xml", self.environment.conf_file)

    def test_get_log_file(self):
        self.assertEquals(TEST_DIR_NAME + "/log.out", self.environment.log_file)

    def test_get_temp_dir(self):
        self.assertEquals(TEST_DIR_NAME + "/tmp", self.environment.temp_dir)

    def test_get_gpx_dir(self):
        self.assertEquals(TEST_DIR_NAME + "/gpx", self.environment.gpx_dir)

    def test_get_extension_dir(self):
        self.assertEquals(TEST_DIR_NAME + "/extensions", self.environment.extension_dir)

    def test_get_plugin_dir(self):
        self.assertEquals(TEST_DIR_NAME + "/plugins", self.environment.plugin_dir)
        

if __name__ == "__main__":
    unittest.main()



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Jakinbidea & Grupo Ikusnet Developer
# vud1@grupoikusnet.com
# Heavily modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import gtk
import pytrainer.lib.webUtils

class About:
    def __init__(self,data_path = None, version = None):
        def url_hook(dialog, url):
            pytrainer.lib.webUtils.open_url_in_browser(url)
        # Available in PyGTK 2.6 and above
        gtk.about_dialog_set_url_hook(url_hook)     
        self.data_path = data_path
        self.version = version

    def run(self):
        authors = ["Fiz Vázquez <vud1@sindominio.net>\nDavid García Granda <dgranda@gmail.com>\nJohn Blance <john.blance@gmail.com>\nArnd Zapletal <a.zapletal@gmail.com>\nNathan Jones <nathan@ncjones.com>\nArto Jantunen <viiru@iki.fi>\n\n-Package maintainers:\n\nRedHat/Fedora: Douglas E. Warner <silfreed@silfreed.net>\nDebian: Noèl Köthe <noel@debian.org>, Christian Perrier <bubulle@debian.org>\nUbuntu: Kevin Dwyer <kevin@pheared.net>, Alessio Treglia <quadrispro@ubuntu.com>"]
        translator_credits = "Euskara: Jabier Santamaria <mendikote@gmail.com>\nCatalà: Eloi Crespillo Itchart <eloi@ikuszen.com>\nČesky: Luboš Pokorny <sp.pok@seznam.cz>\nFrançais: Dj <dj@djremixtheblog.be>, Pierre Gaigé <pgaige@free.fr>, Christian Perrier <bubulle@debian.org>\nNorsk: Havard Davidsen <havard.davidsen@gmail.com>\nPolski: Seweryn Kokot <skokot@po.opole.pl>\nDeutsch: Aleks <aleks@schnecklecker.de>, Noèl Köthe <noel@debian.org>, Erik Pfannenstein <debianignatz@gmx.de>, Andreas Mair <amair.sob@googlemail.com>\nEspañol: Fiz Vázquez <vud1@sindominio.net>, David García Granda <dgranda@gmail.com>\nSvenska: Per Medin <per.medin@gmail.com>\nGalego: Kaptan <khanyux@gmail.com>\nPortuguês: Pedro Ribeiro <p.m42.ribeiro@gmail.com>"
        license = "pytrainer - The free sport tracking center\nCopyright (C) 2005-09 Fiz Vázquez\n\nThis program is free software; you can redistribute it and/or modify\nit under the terms of the GNU General Public License as published by\nthe Free Software Foundation; either version 2 of the License, or\n(at your option) any later version.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with this program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA"
        about_dialog = gtk.AboutDialog()
        about_dialog.set_destroy_with_parent(True)
        about_dialog.set_name("pytrainer")
        about_dialog.set_version(self.version)
        about_dialog.set_copyright("Copyright \xc2\xa9 2005-09 Fiz Vázquez")
        about_dialog.set_website("https://github.com/pytrainer/pytrainer")
        about_dialog.set_website_label("https://github.com/pytrainer/pytrainer")
        about_dialog.set_comments("The free sport tracking center")
        about_dialog.set_license(license)
        
        about_dialog.set_authors(authors)
        about_dialog.set_translator_credits(translator_credits)
        about_dialog.set_logo(gtk.gdk.pixbuf_new_from_file(self.data_path+"glade/pytrainer_mini.png"))

        # callbacks for destroying the dialog
        def close(dialog, response, editor):
            editor.about_dialog = None
            dialog.destroy()
        def delete_event(dialog, event, editor):
            editor.about_dialog = None
            return True
        
        about_dialog.connect("response", close, self)
        about_dialog.connect("delete-event", delete_event, self)  
        self.about_dialog = about_dialog
        about_dialog.show()
        
    def present(self):
        if self.about_dialog is None:
            self.run()
        self.about_dialog.present()



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from migrate.versioning.api import db_version, upgrade, version, version_control
from sqlalchemy.exc import NoSuchTableError
# sqlalchemy-migrate 0.6.1 broke backwards compatibility
# so we need to try importing exceptions from one of two packages
try:
    from migrate.exceptions import DatabaseNotControlledError
except:
    from migrate.versioning.exceptions import DatabaseNotControlledError
from sqlalchemy.schema import MetaData
import logging
import sqlalchemy
import sys

class MigratableDb(object):
    """Object bridge for sqlalchemy-migrate API functions."""
    
    def __init__(self, repository_path, db_url):
        """Create a migratable DB.
        
        Arguments:
        repository_path -- The path to the migrate repository, relative to the
            pypath.
        db_url -- the connection URL string for the DB.
        """
        self._repository_path = repository_path
        self._db_url = db_url
        
    def is_empty(self):
        """Check if the DB schema is empty.
        
        An empty schema indicates a new uninitialised database."""
        metadata = MetaData()
        metadata.bind = sqlalchemy.create_engine(self._db_url)
        metadata.reflect()
        tables = metadata.tables
        return not tables
        
    def is_versioned(self):
        """ Check if the DB has been initiaized with version metadata."""
        try:
            self.get_version()
        except DatabaseNotControlledError:
            return False
        except NoSuchTableError:
            # prior to 0.6.0, sqlalchemy-migrate did not handle querying
            # against unversioned databases.
            return False
        return True
        
    def get_version(self):
        """Get the current version of the versioned DB.
        
        Raises DatabaseNotControlledError if the DB is not initialized."""
        return db_version(self._db_url, self._repository_path)
    
    def get_upgrade_version(self):
        """Get the latest version available in upgrade repository."""
        return version(self._repository_path)
    
    def version(self, initial_version):
        """Initialize the database with migrate metadata.
        
        Raises DatabaseAlreadyControlledError if the DB is already initialized."""
        version_control(self._db_url, self._repository_path, initial_version)

    def upgrade(self):
        """Run all available upgrade scripts for the repository."""
        upgrade(self._db_url, self._repository_path)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import matplotlib
#matplotlib.use('GTK')
from matplotlib.figure import Figure
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvasGTK
#from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTKAgg as NavigationToolbar
import matplotlib.pyplot as plt
#import pylab
import logging
from pytrainer.lib.uc import UC

class DrawGraph:
    def __init__(self, parent = None, pytrainer_main = None):
        logging.debug('>>')
        self.parent = parent
        self.pytrainer_main = pytrainer_main
        self.uc = UC()
        #self.NEARLY_ZERO = 0.0000000000000000000001
        self.ax1 = None
        self.ax2 = None
        logging.debug('<<')

    def draw(self, datalist=None, box=None, figure=None, title=None, y2=False, xgrid=False, ygrid=False):
        '''
            Draw a graph using supplied information into supplied gtk.box

            datalist = populated graphdata class (required)
            box = gtk.box object (required)
            figure = matplotlib figure (optional) if supplied will add graph to this figure
            title =
            y2 =

            return = figure
        '''
        logging.debug('>>')
        if box is None:
            logging.error("Must supply a vbox or hbox to display the graph")
            return
        #Check if have a graph object
        if figure is None:
            #No figure, so create figure
            figure = plt.figure()
            self.ax1 = plt.axes()
            #Reset second axis
            self.ax2 = None
        #Remove any existing plots
        for child in box.get_children():
            logging.debug('Removing box child: '+str(child))
            box.remove(child)

        if datalist is None:
            logging.debug("drawPlot called with no data")
            return figure

        if y2 and self.ax2 is None:
            self.ax2 = plt.twinx()


        #Create canvas
        canvas = FigureCanvasGTK(figure) # a gtk.DrawingArea
        canvas.show()

        #Display title etc
        if datalist.xlabel is not None:
            plt.xlabel(datalist.xlabel)
        if title is not None:
            plt.title(title)
        #Display grid
        if y2 and ygrid:
            self.ax2.grid(True)
        elif self.ax1 and ygrid:
            self.ax1.grid(True)
        plt.gca().xaxis.grid(xgrid)
        #Removed as now in legend
        #plt.ylabel(datalist.ylabel)

        #Determine graph type....
        #print "Got graphtype: %s" % datalist.graphType
        #print datalist.x_values
        #print datalist.y_values
        #print datalist.linewidth
        #print datalist.linecolor
        #print datalist.ylabel
        if datalist.graphType == "plot":
            #Plot data
            if not y2:
                #plt.plot(datalist.x_values, datalist.y_values, linewidth=datalist.linewidth, color=datalist.linecolor, label=datalist.ylabel )
                self.ax1.plot(datalist.x_values, datalist.y_values, linewidth=datalist.linewidth, color=datalist.linecolor, label=datalist.ylabel )
            else:
                self.ax2.plot(datalist.x_values, datalist.y_values, linewidth=datalist.linewidth, color=datalist.y2linecolor, label=datalist.ylabel )
        elif datalist.graphType == "bar":
            if not y2:
                self.ax1.bar(datalist.x_values, datalist.y_values, datalist.bar_widths, datalist.bar_bottoms, color=datalist.linecolor, label=datalist.ylabel, alpha=0.5)
            else:
                self.ax2.bar(datalist.x_values, datalist.y_values, datalist.bar_widths, datalist.bar_bottoms, color=datalist.y2linecolor, label=datalist.ylabel, alpha=0.5)
        elif datalist.graphType == "vspan":
            i = 0
            while i < len(datalist.x_values):
                #print datalist.x_values[i] , datalist.bar_widths[i]
                if not y2:
                    self.ax1.axvspan(datalist.x_values[i], datalist.x_values[i]+datalist.bar_widths[i], alpha=0.15, facecolor=datalist.linecolor)
                else:
                    self.ax2.axvspan(datalist.x_values[i], datalist.x_values[i]+datalist.bar_widths[i], alpha=0.15, facecolor=datalist.y2linecolor)
                i += 1
        elif datalist.graphType == "hspan":
            i = 0
            while i < len(datalist.x_values):
                #print datalist.x_values[i] , datalist.y_values[i], datalist.labels[i], datalist.colors[i]
                if not y2:
                    self.ax1.axhspan(datalist.x_values[i], datalist.y_values[i], alpha=0.25, facecolor=datalist.colors[i], label=datalist.labels[i])
                else:
                    self.ax2.axhspan(datalist.x_values[i], datalist.y_values[i], alpha=0.25, facecolor=datalist.colors[i], label=datalist.labels[i])
                i += 1
        elif datalist.graphType == "date":
            if not y2:
                self.ax1.plot_date(datalist.x_values, datalist.y_values, color=datalist.linecolor, label=datalist.ylabel, alpha=0.5)
            else:
                self.ax2.plot_date(datalist.x_values, datalist.y_values, color=datalist.y2linecolor, label=datalist.ylabel, alpha=0.5)
        else:
            print "Unknown/unimplemented graph type: %s" % datalist.graphType
            return figure
        #Set axis limits
        #plt.axis([datalist.min_x_value, datalist.max_x_value, datalist.min_y_value, datalist.max_y_value])
        if self.ax1 is not None:
            self.ax1.legend(loc = 'upper left', bbox_to_anchor = (0, 1))
        if self.ax2 is not None:
            self.ax2.legend(loc = 'upper right', bbox_to_anchor = (1, 1))
        #axis.set_xlim(0, data.max_x_value)
        #axis.set_ylim(0, data.max_y_value)

        #Display plot
        box.pack_start(canvas, True, True)

        logging.debug("<<")
        return figure

    def drawAthleteGraph(self, athlete = None, box = None):
        '''
            Draw a plot style graph
        '''
        logging.debug('>>')
        #TODO - supply athlete class (and have that populated with all data required...
        if box is None:
            logging.error("Must supply a vbox or hbox to display the graph")
            return
        if athlete is None: # or len(datalist) == 0:
            logging.error("Must supply data to graph graph")
            return
        figure = None
        #Debug info - to remove
        #print("drawPlot....")
        for item in athlete.graphdata:
            #print "drawing", item
            #print athlete.graphdata[item]
            figure = self.draw(athlete.graphdata[item], box=box, figure=figure, title=_("Athlete Data"), y2=athlete.graphdata[item].show_on_y2)

    def drawActivityGraph(self, activity = None, box = None):
        '''
            Draw a multiple style graph using data in an activity (with multiple traces on each axis)
        '''
        logging.debug('>>')
        if box is None:
            logging.error("Must supply a vbox or hbox to display the graph")
            return
        if activity is None:
            logging.error("Must supply data to graph graph")
            return
        #TODO Check that datalist is of type dict (and contains has correct items)
        figure = None
        datalist = []
        y1count = 0
        y2count = 0

        if activity.x_axis == "distance":
            if activity.title is None or activity.title == "":
                _title = "%s %s of %s on %s" % (str(activity.get_value_f('distance', "%0.2f")), self.uc.unit_distance, activity.sport_name, activity.date)
            else:
                _title = "%s: %s %s of %s on %s" % (activity.title, str(activity.get_value_f('distance', "%0.2f")), self.uc.unit_distance, activity.sport_name, activity.date)

            #Loop through data items and graph the selected ones
            for item in activity.distance_data:
                if activity.distance_data[item].show_on_y1:
                    y1count += 1
                    figure = self.draw(activity.distance_data[item], box=box, figure=figure, title=_title, xgrid=activity.x_grid, ygrid=activity.y1_grid)
                if activity.distance_data[item].show_on_y2:
                    y2count += 1
                    figure = self.draw(activity.distance_data[item], box=box, figure=figure, title=_title, y2=True, xgrid=activity.x_grid, ygrid=activity.y2_grid)
            #Display lap divisions if required
            if activity.show_laps:
                figure = self.draw(activity.lap_distance, box=box, figure=figure)

        elif activity.x_axis == "time":
            _time = "%d:%02d:%02d" % (activity.time_tuple)
            if activity.title is None or activity.title == "":
                _title = "%s of %s on %s" % (_time, activity.sport_name, activity.date)
            else:
                _title = "%s: %s of %s on %s" % (activity.title, _time, activity.sport_name, activity.date)
            for item in activity.time_data:
                if activity.time_data[item].show_on_y1:
                    y1count += 1
                    figure = self.draw(activity.time_data[item], box=box, figure=figure, title=_title, xgrid=activity.x_grid, ygrid=activity.y1_grid)
                if activity.time_data[item].show_on_y2:
                    y2count += 1
                    figure = self.draw(activity.time_data[item], box=box, figure=figure, title=_title, y2=True, xgrid=activity.x_grid, ygrid=activity.y2_grid)
            #Display lap divisions if required
            if activity.show_laps:
                figure = self.draw(activity.lap_time, box=box, figure=figure)

        #Sort out graph errors...
        if y1count == 0 and y2count == 0:
            logging.debug("No items to graph.. Removing graph")
            figure = self.draw(None, box=box, figure=figure)
        elif y1count == 0:
            logging.debug("No items on y1 axis... ")
            #TODO Sort
        #Get axis limits..
        if self.ax1 is not None:
            activity.x_limits = self.ax1.get_xlim()
            activity.y1_limits = self.ax1.get_ylim()
        else:
            activity.y1_limits = (None, None)
        if self.ax2 is not None:
            activity.x_limits = self.ax2.get_xlim()
            activity.y2_limits = self.ax2.get_ylim()
        else:
            activity.y2_limits = (None, None)
        #Set axis limits if requested
        #X Axis
        if activity.x_limits_u[0] is not None:
            if self.ax1 is not None:
                self.ax1.set_xlim(activity.x_limits_u)
            elif self.ax2 is not None:
                self.ax2.set_xlim(activity.x_limits_u)
        #Y1 Axis
        if activity.y1_limits_u[0] is not None:
            if self.ax1 is not None:
                self.ax1.set_ylim(activity.y1_limits_u)
        #Y2 Axis
        if activity.y2_limits_u[0] is not None:
            if self.ax2 is not None:
                self.ax2.set_ylim(activity.y2_limits_u)

        return activity
        logging.debug('<<')



#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from datetime import date
from mock import Mock

# Gettext needs to be initialized before UC is imported (via profile)
import pytrainer.lib.localization
pytrainer.lib.localization.initialize_gettext("../../locale")

from pytrainer.lib.ddbb import DDBB
from pytrainer.profile import Profile
from pytrainer.athlete import Athlete

class AthleteTest(unittest.TestCase):

    def setUp(self):
        profile = Mock()
        profile.getValue = Mock(return_value='memory')
        self.ddbb = DDBB(profile)
        main = Mock()
        main.ddbb = self.ddbb
        main.profile = Profile()
        main.ddbb.connect()
        main.ddbb.create_tables(add_default=False)
        self.athlete = Athlete(parent=main)

    def tearDown(self):
        self.athlete = None
        self.ddbb.disconnect()
        self.ddbb = None

    def test_athlete_insert_and_get(self):
        data = {'date': date(2017, 4, 3), 'weight': 60.0, 'bodyfat': 20.0,
                'restinghr': 60, 'maxhr': 190, 'id_athletestat': 1}
        self.athlete.insert_athlete_stats(str(data['date']), data['weight'],
                                          data['bodyfat'], data['restinghr'],
                                          data['maxhr'])
        data2 = self.athlete.get_athlete_stats()
        self.assertEquals(data, data2[0])

    def test_athlete_update_and_get(self):
        data = {'date': date(2017, 4, 3), 'weight': 60.0, 'bodyfat': 20.0,
                'restinghr': 60, 'maxhr': 190, 'id_athletestat': 1}
        self.athlete.insert_athlete_stats(str(data['date']), data['weight'],
                                          data['bodyfat'], data['restinghr'],
                                          data['maxhr'])
        data['maxhr'] = 180
        data['bodyfat'] = 30.0
        self.athlete.update_athlete_stats(1, str(data['date']), data['weight'],
                                          data['bodyfat'], data['restinghr'],
                                          data['maxhr'])
        data2 = self.athlete.get_athlete_stats()
        self.assertEquals(data, data2[0])

    def test_athlete_delete_record(self):
        data = {'date': date(2017, 4, 3), 'weight': 60.0, 'bodyfat': 20.0,
                'restinghr': 60, 'maxhr': 190, 'id_athletestat': 1}
        self.athlete.insert_athlete_stats(str(data['date']), data['weight'],
                                          data['bodyfat'], data['restinghr'],
                                          data['maxhr'])
        self.athlete.delete_record(1)
        self.assertFalse(self.athlete.get_athlete_stats())



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from lib.fileUtils import fileUtils
from pytrainer.gui.dialogs import save_file_chooser_dialog
import logging
import traceback

class Save:
    def __init__(self, data_path = None, record = None):
        self.record = record
        self.data_path = data_path

    def run(self):
        logging.debug('>>')
        filename = save_file_chooser_dialog(title="savecsvfile", pattern="*.csv")
        records = self.record.getAllrecord()
        # CSV Header
        content = "date_time_local,title,sports.name,distance,duration,average,maxspeed,pace,maxpace,beats,maxbeats,calories,upositive,unegative,comments\n"
        try:
            for record in records:
                line = ""
                for i, data in enumerate(record):
                    if i in [3, 5, 6, 7, 8, 12, 13]:
                        try:
                            data = round(data, 2)
                        except:
                            pass             
                    data = "%s" %data
                    data = data.replace(",", " ")  
                    data = data.replace("\n", " ")             
                    data = data.replace("\r", " ")          
                    if i>0: 
                        line += ",%s" %data
                    else:
                        line += "%s" %data      
                content += "%s\n" %line
            logging.info("Record data successfully retrieved. Choosing file to save it")
            file = fileUtils(filename,content)
            file.run()
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
        logging.debug('<<')




# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

class UpgradeContext(object):
    
    """Context used by upgrade scripts.
    
    Provides access to the application base dir."""
    
    def __init__(self, conf_dir, db_url):
        self.conf_dir = conf_dir
        self.db_url = db_url
    
# sqlalchemy-migrate does not provide any means to inject the context object in
# to upgrade scripts so instead we provide access via this global which must be
# initialised before upgrading.
UPGRADE_CONTEXT = None



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from SimpleGladeApp import SimpleBuilderApp
import gtk
import gobject
import os

class WindowExtensions(SimpleBuilderApp):
	def __init__(self, data_path = None, parent=None):
		self.parent = parent
		SimpleBuilderApp.__init__(self, "extensions.ui")

	def new(self):
		column_names=["id","name"]
		self.create_treeview(self.extensionsTree,column_names)

	def setList(self, list):
		iterOne = False
		store = gtk.ListStore(
			gobject.TYPE_STRING,
			gobject.TYPE_STRING
			)
		for i in list:
			iter = store.append()
			if not iterOne:
				iterOne = iter
			store.set (iter, 
				0, i[0],
				1, i[1]
				)
		self.extensionsTree.set_model(store)
		if iterOne:
			self.extensionsTree.get_selection().select_iter(iterOne)
		self.on_extensionsTree_clicked(None,None)
	
	def create_treeview(self,treeview,column_names):
		i=0
		for column_index, column_name in enumerate(column_names):
			column = gtk.TreeViewColumn(column_name, gtk.CellRendererText(), text=column_index)
			if i==0:
				column.set_visible(False)
			treeview.append_column(column)
			i+=1

	def on_extensionsTree_clicked(self,widget,widget2):
		selected,iter = self.extensionsTree.get_selection().get_selected()
		name,description,status,helpfile,type = self.parent.getExtensionInfo(selected.get_value(iter,0))
		self.nameEntry.set_text(name)
		self.descriptionEntry.set_text(description)
		if status is None or int(status) == 0:
			self.statusEntry.set_text(_("Disable"))
		else:
			self.statusEntry.set_text(_("Enable"))

	def on_preferences_clicked(self,widget):
		selected,iter = self.extensionsTree.get_selection().get_selected()
		name,description,status,helpfile,type = self.parent.getExtensionInfo(selected.get_value(iter,0))
		prefs = self.parent.getExtensionConfParams(selected.get_value(iter,0))
		
		self.prefwindow = gtk.Window()
		self.prefwindow.set_border_width(20)
  	    	self.prefwindow.set_title(_("%s settings" %name))
          	
		table = gtk.Table(1,2)
		i=0
		self.entryList = []
		#print prefs
		for key in prefs.keys():
			#print key, prefs[key]
			label = gtk.Label("<b>%s</b>"%key)
			label.set_use_markup(True)
			if key != "status":
				entry = gtk.Entry()
				if prefs[key] is None:
					entry.set_text("")
				else:
					entry.set_text(prefs[key])
				self.entryList.append(entry)	
				table.attach(entry,1,2,i,i+1)
			else:
				combobox = gtk.combo_box_new_text()
				combobox.append_text("Disable")	
				combobox.append_text("Enable")	
				if prefs[key] is None:
					combobox.set_active(0)
				else:
					combobox.set_active(int(prefs[key]))
				table.attach(combobox,1,2,i,i+1)
				self.entryList.append(combobox)	
			table.attach(label,0,1,i,i+1)
			i+=1
				
		button = gtk.Button(_("OK"))
		button.connect("clicked", self.on_acceptSettings_clicked, None)
		table.attach(button,0,2,i,i+1)
          	self.prefwindow.add(table)
          	self.prefwindow.show_all()
	
	def on_help_clicked(self,widget):
		selected,iter = self.extensionsTree.get_selection().get_selected()
		name,description,status,helpfile,type = self.parent.getExtensionInfo(selected.get_value(iter,0))
		file = os.open(helpfile,0)
		text = os.read(file,2000)
		os.close(file)
		helpwindow = gtk.Window()
		button = gtk.Button(_("OK"))
		button.connect("clicked", self.on_accepthelp_clicked, helpwindow)
		vbox = gtk.VBox()
		buffer = gtk.TextBuffer()
		buffer.set_text(text)
		textview = gtk.TextView()
		textview.set_buffer(buffer)
		scrolledwindow = gtk.ScrolledWindow()
		scrolledwindow.add(textview)
		vbox.pack_start(scrolledwindow, True)
		vbox.pack_start(button, False)
          	helpwindow.add(vbox)
		helpwindow.resize(550,300)
          	helpwindow.show_all()
	
	def on_accepthelp_clicked(self,widget,window):
		window.hide()
		window = None

	def on_acceptSettings_clicked(self, widget, widget2):
		selected,iter = self.extensionsTree.get_selection().get_selected()
		prefs = self.parent.getExtensionConfParams(selected.get_value(iter,0))
		savedOptions = []
		i = 0
		for key in prefs.keys():
			try:
				savedOptions.append((key,self.entryList[i].get_text()))
			except:
				combobox = self.entryList[i]
        			index = combobox.get_active()
				savedOptions.append((key,"%s" %index))
			i+=1
		self.prefwindow.hide()
		self.prefwindow = None
		self.parent.setExtensionConfParams(selected.get_value(iter,0),savedOptions)
		self.on_extensionsTree_clicked(None,None)

	def on_accept_clicked(self,widget):
		self.extensions.hide()
		self.extensions = None
		self.quit()
